import glob, os, pickle 
from fs.sshfs import SSHFS
from fs.osfs import OSFS

from .iolib    import *
from .classlib import *
from .clean_files import *

import km3pipe

#Produces run_dic, a dictionary of all bad runs, and interesting runs (such as death runs)
#dominfo_dic, a dictionary of all doms, containing information on their status over runs

import platform
import getpass
print(platform.python_version())

class crawler:
    
    def  __init__(self,
                  JRA_DIR = "/sps/km3net/repo/data/raw/quality/root/",
                  verbose = 0):

        self.verbose = verbose
        self.JRA_DIR = JRA_DIR

        if os.path.isdir( self.JRA_DIR ):
            self.my_fs = OSFS( self.JRA_DIR )
        else:
            self.my_fs = SSHFS( host = "cca.in2p3.fr",
                                user = input( "Lyon username: " ),
                                passwd = getpass.getpass(),
                                timeout = 60)
        
        self.run_dic = {}
        self.dominfo_dic = {}    # key will be a (line,floor) tupledominfo_dic = {}

        if verbose >= 2: print( "Fetching database streams...", end='' )
        self.sds = km3pipe.db.StreamDS()
        if verbose >= 2: print( " done." )

        if verbose >= 2: print( "Made crawler." )
        

    def crawl( self,
               detector,
               OUTRUNS,
               OUTDOMS,
               death_threshold   = 10,  #minimum amount of dead runs to be counted as "dead"
               coma_threshold    = 30,   #minimum amount of dead runs to be counted as "coma"
               passout_threshold = 1 ): #minimum amount of dead runs to be counted as "passout"

        files = {}
            
        for dir_detector in sorted( self.my_fs.listdir( '.' ) ):
            if( detector in DETID_to_OID( dir_detector, self.sds ) ) or ( dir_detector in detector ):
                if self.verbose >= 2: print(" found detector directory: {0}".format(dir_detector))
                files[dir_detector] = []
                for f in self.my_fs.glob( "{}/*/KM3NeT*root".format( dir_detector ) ):
                    files[dir_detector].append( f.path )
                self.run_dic['goodruns']  = [] #list of good JRA runs
                self.run_dic['badruns']   = [] #list of bad JRA runs
                self.run_dic['comaruns']  = {} #dictionary of key: run, value: list of omkeys of comadoms
                self.run_dic['deathruns'] = {} #dictionary of key: run, value: list of omkeys of deathdoms


        if not len( files.items() ):
            if self.verbose >= 1: print("ERROR: no files found.")
            raise ValueError

        # read the jra files
        for det, fdic in files.items():
            print( "detector {0}, total {1} files".format( det, len( fdic ) ) )
            for i, jrafile in enumerate( fdic ):
                if self.verbose >= 2: print( "detector {} file {} of {}".format( det, i, len(fdic) ) )
                process_jra_file( self.JRA_DIR + jrafile, det,
                                  self.run_dic,
                                  self.dominfo_dic,
                                  self.verbose )

        R = getruns( self.dominfo_dic )[-passout_threshold]
        print( "\nThe following were once alive, but are dead in the last {0} runs; i.e since at least run {1}:".format( passout_threshold, R.string() ) )




        #Categorize runs
        for dom_info in sorted( self.dominfo_dic.values(), key = lambda obj: obj.index ):

            dom_info.init_status_runs( death_threshold = death_threshold,
                                       passout_threshold = passout_threshold )
            if dom_info.is_empty():
                print( "DU{0}DOM{1}, empty".format(dom_info.du, dom_info.floor) )
                continue
            if not len( dom_info.live_runs ) :
                print( "DU{0}DOM{1}, always dead".format(dom_info.du, dom_info.floor) )
                continue
            try:
                if last_run( dom_info.live_runs ).index < R.index :
                    death_run = dom_info.get_death_run( death_threshold )
                    print( "DU{0}DOM{1}, dead since {2}".format(dom_info.du, dom_info.floor, death_run.string() ) )
                    death_run.add_note( "DU{0}DOM{1} death".format( dom_info.du, dom_info.floor ) )
                    if death_run in self.run_dic['deathruns']:
                        self.run_dic['deathruns'][death_run].append( (dom_info.du, dom_info.floor) )
                    else:
                        self.run_dic['deathruns'][death_run] = [ (dom_info.du, dom_info.floor) ]

            except AttributeError as ae:
                print( "ERROR: {0}".format( ae ) )

            #Define coma as a very long passout
            coma_runs = dom_info.get_passout_runs( coma_threshold )
            for coma_run in coma_runs:
                if coma_run in self.run_dic['comaruns']:
                    self.run_dic['comaruns'][coma_run].append( (dom_info.du, dom_info.floor) )
                else:
                    self.run_dic['comaruns'][coma_run] = [ (dom_info.du, dom_info.floor) ]
                    
        outfile_runs = open( OUTRUNS, 'wb')
        outfile_doms = open( OUTDOMS, 'wb')
        pickle.dump( self.run_dic, outfile_runs )
        pickle.dump( self.dominfo_dic, outfile_doms )
        outfile_runs.close()
        outfile_doms.close()

        return 0
        
    def pront_dominfo( dominfo_dic ):
        pront( dominfo_dic )

    def pront( self ):
        print( "runs with bad jra files : ", [ obj.string() for obj in self.run_dic['badruns'  ] ] )
        print( "runs with coma : ",          [ obj.string() for obj in self.run_dic['comaruns' ] ] )
        print( "runs with deaths : ",        [ obj.string() for obj in self.run_dic['deathruns'] ] )
        print( "dead  in all runs : ", sorted( [ d for d in self.dominfo_dic if not len( self.dominfo_dic[d].live_runs ) ] ) )
        print( "alive in all runs : ", sorted( [ d for d in self.dominfo_dic if self.dominfo_dic[d].alive_in_runs( getruns( self.dominfo_dic ) ) ]) )



#Plotting functions

import os, math, copy, glob
import matplotlib.dates as md
import matplotlib.pyplot as plt
import numpy as np

from .iolib import *
from .classlib import *

N_FLOORS = 19

def handle_fig( fig,
                name,
                save = True,
                show = False,
                dpi = None,
                dimensions = None,
                labelsize  = 'medium',
                titlesize  = 'medium',
                legendsize = 'medium',
                xticksize  = 'small',
                yticksize  = 'small',
                VERBOSE = False): #takes tuple
    if dimensions: fig.set_size_inches( dimensions )
    plt.tight_layout(pad=3.0)

    plt.rcParams.update({'axes.labelsize' : labelsize,
                         'axes.titlesize' : titlesize,
                         'legend.fontsize': legendsize,
                         'xtick.labelsize': xticksize,
                         'ytick.labelsize': yticksize})

    if save:
        for img_format in ['png', 'pdf']:
            plt.savefig( name + '.' + img_format,
                         dpi = dpi,
                         format=img_format)
            if VERBOSE: print( "Saved {0}".format( name + '.' + img_format ) )
    if show:
        plt.draw()
        plt.show()

    plt.cla()



            

def plot_scp_DOM_hist( scp_datagroup,
                       ax,
                       clb,
                       XRANGE,
                       XBINS = 100,
                       YBINS = 100,
                       title = '',
                       x_label = '',
                       y_label = '',
                       accent = False,
                       DATE_FORMAT = "%d/%m/%Y %H:%M"):
    """
    Plot histogram plot of a DOM in ax
    """
    
    raw_data_values = scp_datagroup.loc[:, 'DATA_VALUE'].values
    data_values = np.array([ value for value in raw_data_values if str(value) != 'nan' ] )
    data_min           = min(data_values)
    data_max           = max(data_values)
    data_values_mean   = data_values.mean()
    data_values_spread = data_values.std()
    if not data_values_spread: data_values_spread = 1.
    data_delta         = get_min_diff( data_values )
    if not data_delta: data_delta = 1.
    y_min   = data_min - data_values_spread / 2.
    y_max   = data_max + data_values_spread / 2.
    y_delta = y_max - y_min 
    y_bins  = np.ceil( YBINS if  y_delta / data_delta > YBINS else y_delta / data_delta )

    ax.hist2d( scp_datagroup.loc[:, 'UNIXTIME'].values,
               raw_data_values,
               bins  = [XBINS, y_bins],
               range = [XRANGE, [y_min, y_max]])
    col = ['black', 'red'][accent]
    ax.set_title( title, color = col)
    if x_label: ax.set_xlabel( x_label, color = col)
    if y_label: ax.set_ylabel( y_label, color = col)
    ax.xaxis_date()
    ax.xaxis.set_major_formatter(md.DateFormatter( DATE_FORMAT ) )
    ax.set_xticks( get_run_starts( scp_datagroup ))
    ax.tick_params( axis = 'x', rotation = 8 )

    return 1

def plot_scp_DOM_scat( scp_datagroup,
                       ax,
                       clb,
                       XRANGE = None,
                       title = '',
                       x_label = '',
                       y_label = '',
                       accent = False,
                       DATE_FORMAT = "%d/%m/%Y %H:%M"):
    """
    Plot scatter plot of a DOM in ax
    """

    colormap = lambda i: plt.cm.gnuplot2(i/19.)
    scp_datagroup.plot(x = 'UNIXTIME',
                       y = 'DATA_VALUE',
                       ax = ax,
                       style=['.', '*'][accent],
                       legend=False,
                       color=[colormap(clb.floor), 'red'][accent],           
                       xlim = XRANGE )
    col = ['black', 'red'][accent]
    ax.set_title( title, color = col )
    if x_label: ax.set_xlabel( x_label, color = col )
    if y_label: ax.set_ylabel( y_label, color = col )
    ax.xaxis_date()
    ax.xaxis.set_major_formatter( md.DateFormatter( DATE_FORMAT ) )
    ax.grid( True, which = 'minor', axis = 'x')
    ax.set_xticks( get_run_starts( scp_datagroup ), minor = True )
    return 1







def plot_scp_DU_hist( scp_datagroup,
                      clbm,
                      fig,
                      ax,
                      XRANGE,
                      XBINS = 100,
                      NORM = False,
                      LOG  = False,
                      title = '',
                      x_label = '',
                      y_label = '',
                      w_label = '',
                      ACCENT_DOMS = None,
                      VERBOSE = False):
    """
    Plot histogram plot of all DOMs in DU in ax
    """
    du = clbm.upis[ scp_datagroup.iloc[0, :].SOURCE_NAME ].du
    DU_floors = [None] * N_FLOORS
    for (idu, ifloor), clb in clbm.omkeys.items():
        if idu == du: DU_floors[ ifloor ] = "{0} ({1})".format( clb.upi, ifloor )


    map_times = np.linspace(*XRANGE, XBINS) 
    
    data_array    = np.empty((XBINS, N_FLOORS))
    data_array[:, :] = None
    norm_array    = np.zeros( data_array.shape )
    
    for index, entry in scp_datagroup.iterrows():

        itime = np.where( entry.get('UNIXTIME')    <= map_times )[0][0]
        clb_upi   = entry.get('SOURCE_NAME')
        clb_floor = clbm.upis[ clb_upi ].floor
        iclb      = clb_floor

        if math.isnan( data_array[itime, iclb] ):
            data_array[itime, iclb] = 0
            norm_array[itime, iclb] = 0
        data_array[itime, iclb] += entry.get('DATA_VALUE')
        norm_array[itime, iclb] += 1.
    
    norm_array[norm_array == 0] = 1.
    data_array /= norm_array

    if NORM:
        for dom_bin in range( data_array.shape[1] ):
            if not np.any( data_array[ :, dom_bin ] == data_array[ :, dom_bin ] ):
                continue
            row = data_array[ :, dom_bin ]
            clean_row = row[ ~np.isnan( row ) ]
            row_mean = np.mean( clean_row ) if np.mean( clean_row ) else 1
            data_array[ :, dom_bin ] /= row_mean

    if LOG:
        data_array = np.ma.log10( data_array )

    pos = ax.imshow( data_array.transpose(),
                     origin = 'lower',
                     aspect = 'auto',
                     extent = [*XRANGE, 0, N_FLOORS] )
    
    ax.set_yticks(np.arange( N_FLOORS ) + 0.5)
    ax.set_yticklabels( DU_floors )
    ax.set_xticks( get_run_starts( scp_datagroup ))
    plt.setp(ax.get_xticklabels(), rotation=10, ha = 'right')
    if ACCENT_DOMS:
        for accent_dom in ACCENT_DOMS:
            ax.get_yticklabels()[ accent_dom ].set_color('red') 

    ax.set_title( title )
    if x_label: ax.set_xlabel( x_label )
    if y_label: ax.set_ylabel( y_label )
    ax.xaxis_date()
    ax.xaxis.set_major_formatter( md.DateFormatter("%d/%m/%Y %H:%M") )

    cbar = fig.colorbar(pos, ax=ax )
    cbar.set_label( w_label, rotation=90)
    #cbar.set_label((["<{0}>", "log10<{0}>"][LOG] + " ({1})").format( PARAM_MEANING, PARAM_UNIT), rotation=90)

def plot_scp_DU_scat( scp_datagroup,
                      DU_omkeys,
                      ax,
                      XRANGE  = None,
                      NORM    = False,
                      LOG     = False,
                      title = '',
                      x_label = '',
                      y_label = '',
                      ACCENT_DOMS = None,
                      VERBOSE = False):
    """
    Plot scatter plot of all DOMs in DU in ax
    """

    colormap = lambda i: plt.cm.gnuplot2(i/19.)
    clbs = DU_omkeys.values()
    if ACCENT_DOMS:
        for accent_dom in ACCENT_DOMS:
            accent_clb = DU_omkeys.pop( ( list( DU_omkeys.keys() )[0][0], accent_dom) )
            clbs = list( DU_omkeys.values() ) + [accent_clb]
    else:
        ACCENT_DOMS = []
        
    for clb in clbs:

        scp_clb_datagroup = scp_datagroup.loc[scp_datagroup.loc[:, 'SOURCE_NAME'] == clb.upi].copy()
        
        if NORM:
            scp_clb_datagroup.loc[:, 'DATA_VALUE'] /= scp_clb_datagroup.DATA_VALUE.mean()
        try:
            scp_clb_datagroup.plot(
                x = 'UNIXTIME',
                y = 'DATA_VALUE',
                ax=ax,
                label = "Floor{1}:{2}".format(clb.du,
                                              clb.floor,
                                              clb.dom_id),
                style=['.', '*'][clb.floor in ACCENT_DOMS],
                ms   = 10, #marker size
                logy = LOG,
                xlim = XRANGE,
                legend=True,
                color=[colormap(clb.floor), 'red'][clb.floor in ACCENT_DOMS])

        except TypeError as e: #no data to plot
            if VERBOSE: print("Error for floor {0}: {1}".format(clb.floor, e))
            continue

    ax.set_title( title )
    if x_label: ax.set_xlabel( x_label )
    if y_label: ax.set_ylabel( y_label )
    ax.xaxis_date()
    ax.xaxis.set_major_formatter( md.DateFormatter("%d/%m/%Y %Hh") )
    ax.legend(loc=3,fontsize=8)
    ax.grid( True, which = 'minor', axis = 'x')
    ax.set_xticks( get_run_starts( scp_datagroup ), minor = True )






def plot_DET_DOMs( scp_dataframe,
                   clbm,
                   MINRUN,
                   MAXRUN,
                   PARAM_NAME,
                   PARAM_UNIT,
                   PARAM_MEANING,
                   OUTPATH,
                   DETECTOR,
                   ACCENT = None, #list of omkeys
                   HIST = True,
                   SCAT = True,
                   VERBOSE = False,
                   SAVE_FIG = True,
                   SHOW_FIG = False):
    """
    Create all DOM plots for a detector
    """

    len_dataframe = scp_dataframe.loc[:, 'SOURCE_NAME'].value_counts()[0]
    x_bins = int( len_dataframe if len_dataframe < 200 else len_dataframe / 20 )

    date_start = np.sort( md.epoch2num( scp_dataframe.loc[:, 'UNIXTIME'].values / 1000.))[0]
    date_end   = np.sort( md.epoch2num( scp_dataframe.loc[:, 'UNIXTIME'].values / 1000.))[-1]

    for idu, du in enumerate( get_live_DUs( scp_dataframe, clbm ) ):
        plt.cla()
        plt.clf()
        dom_fig, dom_axs = plt.subplots(nrows=19, ncols=2)
        if VERBOSE: print("\nDU:{0}".format(du), end='')
        for (omkey, clb) in sorted( clbm.omkeys.items() ):
            if clb.du != du:
                continue
            try:
                scp_datagroup = get_DOM_datagroup( scp_dataframe, clb )
            except ValueError as e: #no data
                if VERBOSE: print(" Error for dom {0}: {1}".format(clb.floor, e))
                continue

            if( scp_datagroup.size < 1 ):
                print("ERROR: datagroup empty! Skipping")
                continue
            if VERBOSE:
                print( "Plotting datagroup of size {0}".format( scp_datagroup.size ) )

            accent_this = False
            if ACCENT and omkey in ACCENT:
                accent_this = True

            if HIST:

                h_title = '{0} for DOM{1} (DU{2} Floor{3}) from run {4} to run {5}'.format(
                    PARAM_MEANING,
                    clb.dom_id,
                    clb.du,
                    clb.floor,
                    MINRUN,
                    MAXRUN)
                hx_label = ''
                hy_label = '{0} ({1})'.format( PARAM_MEANING, PARAM_UNIT )

                plot_scp_DOM_hist( scp_datagroup,
                                   dom_axs[clb.floor, 0],
                                   clb,
                                   XRANGE = [date_start, date_end],
                                   XBINS = x_bins,
                                   title = h_title,
                                   x_label = hx_label,
                                   y_label = hy_label,
                                   accent = accent_this)

            if SCAT:
                s_title  = '{0} for DOM{1} (DU{2} Floor{3}) from run {4} to run {5}'.format(
                    PARAM_MEANING,
                    clb.dom_id,
                    clb.du,
                    clb.floor,
                    MINRUN,
                    MAXRUN)
                sx_label = ''
                sy_label = '{0} ({1})'.format( PARAM_MEANING, PARAM_UNIT )
                plot_scp_DOM_scat( scp_datagroup,
                                   dom_axs[clb.floor, 1],
                                   clb,
                                   [date_start, date_end], #XRANGE
                                   title = s_title,
                                   x_label = sx_label,
                                   y_label = sy_label,
                                   accent = accent_this)
        
        handle_fig( dom_fig,
                    OUTPATH,
                    save = SAVE_FIG,
                    show = SHOW_FIG,
                    dimensions = (35, 65) )

        plt.close( dom_fig )

def plot_DET_DUs( scp_dataframe,
                  clbm,
                  MINRUN,
                  MAXRUN,
                  PARAM_NAME,
                  PARAM_UNIT,
                  PARAM_MEANING,
                  OUTPATH,
                  DETECTOR,
                  ACCENT = None, #list of omkeys
                  HIST = True,
                  SCAT = True,
                  NORM = False,
                  LOG = False,
                  VERBOSE = False,
                  SAVE_FIG = True,
                  SHOW_FIG = False):
    """
    Create all DU plots for a detector
    """

    len_dataframe = scp_dataframe.loc[:, 'SOURCE_NAME'].value_counts()[0]
    x_bins = int( len_dataframe if len_dataframe < 200 else len_dataframe / 20 )

    date_start = np.sort( md.epoch2num( scp_dataframe.loc[:, 'UNIXTIME'].values / 1000.))[0]
    date_end   = np.sort( md.epoch2num( scp_dataframe.loc[:, 'UNIXTIME'].values / 1000.))[-1]

    live_DUs = get_live_DUs( scp_dataframe, clbm )

    du_fig, du_axs = plt.subplots( len( live_DUs ), 2 )
    du_axs.reshape( len( live_DUs), 2 )
    print( live_DUs)
    for idu, du in enumerate( live_DUs ):
        
        if VERBOSE: print("\nDU: {0} ".format(du), end='')                                                                          
        DU_omkeys = get_DU_omkeys( [du], clbm.omkeys, ignore_base = False)
        
        try:
            scp_datagroup = get_DU_datagroup( scp_dataframe, [du], DETECTOR )
        except ValueError as e: #no data
            if VERBOSE: print(" ERROR: DU {0}: {1}. Skipping this DU.".format(du, e))
            continue

        if scp_datagroup.size < 1 :
            if VERBOSE: print(" ERROR: DU {0} datagroup empty! Skipping this DU.".format(du) )
            continue
        if VERBOSE:
            print( "Plotting datagroup of size {0}".format( scp_datagroup.size ) )

        title = '{0} for {1}-DU{2} from run {3} to run {4}'.format(
            PARAM_MEANING,
            DETECTOR,
            du,
            MINRUN,MAXRUN)
        x_label = '',

        p_m = (["{0}", "log10{0}"][LOG]).format( PARAM_MEANING )
        p_u = ([[" ({0})", " (log_10 {0})"][LOG], ""][NORM]).format(PARAM_UNIT)


        if len( du_axs.shape ) == 1:
            #du_axs reshapes to 
            du_ax = du_axs
        else:
            du_ax = du_axs[idu]

        ACCENT_DOMS = None
        if ACCENT and du == ACCENT[0]:
            ACCENT_DOMS = ACCENT[1]

        if HIST:
            hy_label = 'CLB upi (Floor)'
            w_label= ["<" + p_m + ">_bin", "<" + p_m + ">_bin" + "/<" + p_m + ">_DOM"][NORM] + p_u

            plot_scp_DU_hist( scp_datagroup,
                              clbm,
                              du_fig,
                              du_ax[0],
                              [date_start, date_end],
                              XBINS = len_dataframe,
                              NORM = NORM,
                              LOG  = LOG,
                              title   = title,
                              x_label = x_label,
                              y_label = hy_label,
                              w_label = w_label,
                              ACCENT_DOMS = ACCENT_DOMS,
                              VERBOSE = VERBOSE)

        if SCAT:
            sy_label= [p_m, p_m + "/<" + p_m + ">_DOM"][NORM] + p_u
            plot_scp_DU_scat( scp_datagroup,
                              DU_omkeys,
                              du_ax[1],
                              [date_start, date_end],
                              NORM = NORM,
                              LOG  = LOG,
                              title = title,
                              x_label = x_label,
                              y_label = sy_label,
                              ACCENT_DOMS = ACCENT_DOMS,
                              VERBOSE = VERBOSE)
            
    handle_fig( du_fig,
                OUTPATH,
                save = SAVE_FIG,
                show = SHOW_FIG,
                dimensions = (24, 12 * len( live_DUs )),
                labelsize  = 'x-large',
                titlesize  = 'x-large',
                legendsize = 'large',
                xticksize  = 'large',
                yticksize  = 'large' )
    plt.close( du_fig )


    
def plot_status_DU( x,
                    y,
                    w,
                    run_bins,
                    outplot,
                    title = "",
                    accent_runs = [], #list of (run_n, floor) tuples
                    aspect_ratio = 11.5):

    """
    Plot live status of doms for a single DU
    """

    fig, ax = plt.subplots()
    ax.set_aspect( aspect_ratio )

    h = ax.hist2d( x,
                   y,
                   weights = w,
                   bins = [ run_bins, np.arange( 19 + 1 ) ],
                   cmap = 'Set2',
                   vmin = 0, #This fits with the cmap.
                   vmax = 9) #

    x_scale = run_bins[-1] - run_bins[0]
    x_spacing = 10 if x_scale > 100 else 1
    ax.set_xticks(      np.array( run_bins[0::x_spacing] ) + x_spacing / 2 )
    ax.set_xticks(      np.array( run_bins )               +           0.5, minor = True )
    ax.set_xticklabels( np.array( run_bins[0::x_spacing] ).astype( int ) + int( x_spacing / 2 ),
                         rotation=90,
                         ha="right",
                         va="center",
                         rotation_mode="anchor" )

    ax.set_yticks(      np.arange( 19 ) + 0.5) #0.5, ..., 18.5
    ax.set_yticks(      np.arange( 19 ), minor = True) #0, ..., 18
    ax.set_yticklabels( np.arange( 19 ) ) #0, ..., 18
    for run_tuple in accent_runs:
        ax.set_xticks( list( ax.get_xticks() ) + [run_tuple[0] + x_spacing / 2] )
        ax.get_xticklabels()[ int( ( run_tuple[0] - run_bins[0] + 0.5 ) / x_spacing ) ].set_color('red') 
        ax.get_yticklabels()[ run_tuple[1] ].set_color('red') 

    plt.tick_params(
        axis  = 'both',
        which = 'both',
        bottom= True,  # ticks along the bottom edge are on
        top   = False,    # ticks along the top edge are off
        left  = True,
        right = False,
        direction = 'out')


    plt.xlabel("Run n")
    plt.ylabel("Floor #")
    plt.title( title )
    cbar = plt.colorbar( h[3], ticks = [0.5, 1.6, 2.8, 5.1, 6.2, 8.4] )
    cbar.ax.set_yticklabels(['absent', 'death', 'dead', 'healthy', 'passout', 'bad run'])
    plt.grid( axis='x', linewidth='0.1', color = 'black', which = 'major')
    plt.grid( axis='y', linewidth='0.1', color = 'black', which = 'minor')

    handle_fig( fig,
                outplot,
                save = True,
                dpi = 100,
                dimensions = ( ( x_scale + 55 ) / 50,  (19 + 7) * aspect_ratio / 50 ),
                labelsize  = 'large',
                titlesize  = 'large',
                legendsize = 'large',
                xticksize  = 'small',
                yticksize  = 'medium' )
    plt.close( fig )

def plot_status_DET( dom_infos,
                     runlist,
                     outplot_name,
                     badruns = [],
                     det_id = None,
                     title = '',
                     accent = False,
                     force_recreate = False,
                     verbose = 0):
    """
    Call plot_DU for whole detector
    """

    death_threshold = 10
    passout_threshold = 2

    x_times    = []
    y_floors   = []
    w_statuses = []

    run_bins = runlist + [runlist[-1] + 1]
    plot_data = {}

    for (du, floor), dom in dom_infos.items():

        #Get only physics runs for this detector
        det_dom = copy.deepcopy( dom ).filter_det( det_id )
        if not det_dom:
            continue
        phys_dom = det_dom.filter_runs( runlist )
        if not phys_dom:
            continue

        badruns = get_det_id_runs( badruns, det_id )
        phys_dom.init_status_runs( death_threshold,
                                   passout_threshold,
                                   det_id )

        if du not in plot_data:
            plot_data[ du ] = {'x': [],
                               'y': [],
                               'w': []}
            
        healthy_runs = phys_dom.healthy_runs
        passout_runs = phys_dom.passout_runs
        death_run    = phys_dom.death_run
        dead_runs    = phys_dom.dead_runs
        deaths = [] #run_n, floor

        if verbose >= 2:
            print( "DU{0} badruns {1}".format( du, len( badruns ) ), end = '' )
            print( " - DOM{0}, healthy {1}, dead {2}, passout {3}, death {4}".format( floor,
                                                                                      len(healthy_runs),
                                                                                      len(dead_runs),
                                                                                      len(passout_runs),
                                                                                      bool( death_run ) ))

        #blue = dead
        plot_data[ du ]['x'].extend( np.array( get_sorted_run_n( dead_runs ) ) + 0.5 )
        plot_data[ du ]['y'].extend( ( np.ones( len(dead_runs) ) * floor )       + 0.5 )
        plot_data[ du ]['w'].extend(   np.zeros( len(dead_runs) ) + 3 )

        #green = healthy
        plot_data[ du ]['x'].extend( np.array( get_sorted_run_n( healthy_runs ) ) + 0.5 )
        plot_data[ du ]['y'].extend( ( np.ones( len(healthy_runs) ) * floor )       + 0.5 )
        plot_data[ du ]['w'].extend(  np.zeros( len(healthy_runs) ) + 5 )

        #yellow = passout
        plot_data[ du ]['x'].extend( np.array( get_sorted_run_n( passout_runs ) ) + 0.5 )
        plot_data[ du ]['y'].extend( ( np.ones( len(passout_runs) ) * floor ) + 0.5 )
        plot_data[ du ]['w'].extend( np.zeros( len(passout_runs) ) + 6             )


        #orange = death
        if death_run:
            plot_data[ du ]['x'].append( death_run.n + 0.5 )
            plot_data[ du ]['y'].append( floor + 0.5 )
            plot_data[ du ]['w'].append( 2 )
            
            deaths.append( (death_run.n, floor) )

    for du in plot_data:

        #grey = bad run
        plot_data[ du ]['x'].extend( np.repeat( get_sorted_run_n( badruns ), 19 ) + 0.5 )
        plot_data[ du ]['y'].extend( np.tile( np.arange(19), len(badruns) )       + 0.5 )
        plot_data[ du ]['w'].extend( np.zeros( len(badruns) * 19 ) + 9                  )
        
        outplot = outplot_name.replace( "PLOTINFO", "DU%i_runs%i-%i" %( du,
                                                                        min(plot_data[du]['x']),
                                                                        max(plot_data[du]['x']) ) )

        if not len( plot_data[ du ]['x'] ):
            continue

        plot_exists = False
        for plot_file in glob.glob( outplot + "*" ):
            plot_exists = False
            if force_recreate:
                os.remove( plot_file )
            else:
                plot_exists = True
        if plot_exists:
            print( "File already exists, continuing." )
            continue

        print("Making {0}".format( outplot ) )
        if len( plot_data[ du ]['x'] ):
            plot_status_DU( plot_data[ du ]['x'],
                            plot_data[ du ]['y'],
                            plot_data[ du ]['w'],
                            run_bins,
                            outplot,
                            title = title + " DU{0}".format( du ),
                            accent_runs = [ None, deaths ][accent])
        else:
            print( "No data found! Skipping." )




def get_min_diff( array ):
    sorted_array = np.unique( array )
    if len(sorted_array) < 2: return 0 #the array could contain the same value multiple times!
    min_diff = abs( sorted_array[1] - sorted_array[0] )
    for i, j in enumerate( sorted_array ):
        if i == len(sorted_array) - 1:
            break
        min_diff = sorted_array[i + 1] - j if sorted_array[i + 1] - j < min_diff else min_diff
    return min_diff

def get_run_starts( scp_datagroup,
                    VERBOSE  = False):
    run_starts = []
    for runid in scp_datagroup.loc[:, 'RUN'].unique():
        start = sorted( scp_datagroup.loc[ scp_datagroup.loc[:, 'RUN'] == runid, 'UNIXTIME' ].values )[0]
        run_starts.append( start )
    return run_starts



# (det_id, run_n): (du, floor)
RUNDIC = {
    ( KM3NeT_00000014, 3816 ): (1, 18), #died between 3786 and 3816
    ( KM3NeT_00000014, 5103 ): (2, 1 ), #died on 5102 or 5103 https://elog.km3net.de/Operations+IT/3375
    ( KM3NeT_00000014, 5047 ): (2, 16), #died 5047 https://elog.km3net.de/Operations+IT/3326
    ( KM3NeT_00000043, 4227 ): (1, 1 )  #died 4227 https://elog.km3net.de/Operations+FR/1538
}
    #( KM3NeT_00000045, 0    ): (6, 6),  #died at deployment
    #( KM3NeT_00000045, 0    ): (6, 7),  #died at deployment
    #( KM3NeT_00000044, 0    ): (2, 15), #died at deployment https://elog.km3net.de/Operations+FR/2372
    #( KM3NeT_00000044, 0    ): (2, 16)  #died at deployment https://elog.km3net.de/Operations+FR/2372
    



import os, ROOT, aa

# The following puts root in batch mode; you will not see graphics
# output on your screen.
ROOT.gROOT.SetBatch( True )

# Make directory to put the files
dr = "web_scp"
os.system('mkdir ' + dr)

# We will use the WebFile object;
# no need, compile, you can just include the .hh file...
aa.include( os.environ['AADIR']+"/ana/histogrammer/WebFile.hh")
W = ROOT.WebFile( dr )

# make a canvas with some plot
c = ROOT.TCanvas('c','title', 600,600 )
h = ROOT.TH1D("h","hist",100,0,1)
for i in range(101) : h.SetBinContent( i, ( i - 50 )**2 )
h.SetFillColor(5)
h.Draw()


# You can write bare html
W.write("<h1> This is the Title <h1> <br> ")

# You can write a root canvas like so:
W.write( c )
h.SetFillColor(4)
h.Draw()
c.SetName("c2") # this is important since the name of the .png file is based the canvas Name
W.write(c)

# If you have a png / gif you can add it like this:
h.SetFillColor(2)
h.Draw()
c.SaveAs(dr + "/plot.png")     # put it in the webdir
W.write_img_tag( 'plot.png','')

del W # destructor closes the file

# you can now copy the webdir to a webserver -- for example
cmd = "scp -r " + dr + +" jseneca@login.nikhef.nl:~/public_html"
os.system(cmd)
# it will be viewable here : https://www.nikhef.nl/~t61/webdir/






class Run( object ):

    def __init__( self, det_id, n, note = ''):
        self.det_id = det_id
        self.n      = n  
        self.notes  = [note]
        self.index  = self.get_index()

    def __hash__(self):
        return hash( self.index )

    def __eq__(self, other):
        return other and self.index == other.index

    def __ne__(self, other):
        return not other or self.index != other.index

    def __lt__(self, other):
        return self.index < other.index

    def __gt__(self, other):
        return self.index > other.index

    def string( self ):                
        return '{0}_{1}'.format( self.det_id, self.n )

    def get_index( self ): #int value to sort easily
        """
        function to return a string that can be use to nicely sort runs
        """
        return int( self.det_id.split('_')[-1] + str(self.n).zfill(5) )

    def add_note( self, note ):
        self.notes.append( note )
        
    def dump_notes( self, path ):
        outfile = open( path + "/RUN{0}_notes".format( self.n ), 'w')
        for note in self.notes:
            outfile.write( note + "\n" )
        outfile.close()

    def __repr__( self ):
        return "{} - {} - {}".format( self.det_id, self.n, self.notes ) 

    



class DomInfo( object ):

    def __init__( self, du, floor ):
        self.du        = du
        self.floor     = floor
        self.index  = self.get_index()

        self.detectors = []
        self.live_runs = [] #Runs with data
        self.dead_runs = [] #Runs with no data

        self.passout_runs = []   #Live runs followed by temporary death
        self.death_run    = None #Last live run followed by permanent death (only dead runs)
        self.healthy_runs = []   #Live runs which are not passout nor death runs

    def init_status_runs( self,
                          death_threshold,
                          passout_threshold,
                          det_id = None):
        self.death_run    = self.get_death_run(      death_threshold, det_id)
        self.passout_runs = self.get_passout_runs( passout_threshold, det_id) 
        self.healthy_runs = [ run for run in self.live_runs if ( ( not det_id or run.det_id == det_id )
                                                                 and run not in self.passout_runs
                                                                 and run != self.death_run ) ]

    def get_passout_runs(self,
                         threshold = 1,
                         det_id = None):
        self.live_runs = get_time_sorted_runs( self.live_runs )
        passout_runs = []
        for irun, run in enumerate( self.live_runs ):
            if ( not det_id or run.det_id == det_id )\
            and run.index + 1 in get_sorted_indices( self.dead_runs )\
            and run != self.death_run\
            and run.index < self.live_runs[irun + 1].index - threshold:
                passout_runs.append( run )
        return passout_runs

    def get_death_run( self, threshold = 1, det_id = None):
        if not self.dead_runs:
            return None
        last_live_run = last_run( self.live_runs, det_id )
        if not last_live_run:
            return None
        return  last_live_run if last_live_run.index < last_run( self.dead_runs, det_id ).index - threshold else None 

    def is_empty(self):
        return len(self.live_runs) == 0 and len(self.dead_runs) == 0

    def alive_in_runs ( self, runlist ):
        return all( run in self.live_runs for run in runlist )

    def filter_det( self, det_id = None ):
        if det_id == None:
            return self

        self.live_runs = get_time_sorted_runs( self.live_runs, det_id )
        self.dead_runs = get_time_sorted_runs( self.dead_runs, det_id )
        if self.is_empty():
            return None
        
        self.passout_runs = get_time_sorted_runs( self.passout_runs, det_id )
        self.healthy_runs = get_time_sorted_runs( self.healthy_runs, det_id )
        if not self.death_run or self.death_run.det_id != det_id:
            self.death_run = None
        return self

    def filter_runs( self, runlist = [] ):
        if not len( runlist ):
            return self

        self.live_runs = [run for run in self.live_runs if run.n in runlist]
        self.dead_runs = [run for run in self.dead_runs if run.n in runlist]
        if self.is_empty():
            return None
        
        self.passout_runs = [run for run in self.passout_runs if run.n in runlist]
        self.healthy_runs = [run for run in self.passout_runs if run.n in runlist]
        if not self.death_run or self.death_run.det_id not in runlist:
            self.death_run = None
        return self

    def get_index( self ):
        """
        function to return a string that can be use to nicely sort doms
        """
        return int (str( self.du ).zfill( 3 ) + str( self.floor ).zfill( 2 ) )

def get_det_id_runs( runs, det_id ):
    """
    Get the sublist of runs which have det_it
    """
    return [ run for run in runs if run.det_id == det_id ]


def get_sorted_indices( da_objects ):
    """
    Get indices of a list of objects, sorted
    """
    indices = []
    for obj in da_objects:
        indices.append( obj.index )
    return sorted( indices )

def get_sorted_run_n( runs ):
    """
    Get run numbers of a list of runs, sorted
    """
    run_ns = []
    for run in runs:
        run_ns.append( run.n )
    return sorted( run_ns )


def get_time_sorted_runs( runs, det_id = None):
    out_runs = runs
    if det_id:
        out_runs = get_det_id_runs( runs, det_id )
    return sorted( out_runs, key=lambda obj: obj.index )

               
def last_run( runs, det_id = None):
    if runs:
        last_run = get_time_sorted_runs( runs )[-1]
        return last_run if ( not det_id or last_run.det_id == det_id ) else None
    else:
        return None



import os, pickle, glob

from .plotlib import *
from .iolib   import *

import km3pipe

class status_plotter:
    
    def __init__( self,
                  verbose = 0):
        
        self.verbose = verbose
        self.sds = km3pipe.db.StreamDS()

    def plot( self,
              detector,
              domsfile,
              runsfile,
              outpath = '',
              force_recreate = False ):

        try:
            df = open( domsfile, "rb" )
        except IOError:
            print( "ERROR: file {0} could not be found.".format( domsfile ) )
            raise IOError

        try:
            rf = open( runsfile, "rb" )
        except IOError:
            print( "ERROR: file {0} could not be found.".format( runsfile ) )
            raise IOError

        dominfo_dic = pickle.load( df )
        run_dic     = pickle.load( rf )
              
        detectors = get_detectors( detector,
                                   self.sds )

        for det in detectors:
            runtable = self.sds.runs( detid = DETID_to_OID( det, self.sds ) )
            runlist = list( runtable.loc[ runtable.loc[:, 'RUNSETUPNAME'].str.startswith( 'PHYS'), 'RUN' ].values )
            if not len( runlist ):
                continue


            print( "Plotting status of {}".format( det ), end = '')
            try:
                print( "({})...".format( DETID_to_OID( det, self.sds ) ) )
            except ValueError:
                print( " name error..." )

            outplot_name = "{}{}_status_PLOTINFO".format( outpath, DETID_to_OID( det, self.sds ) )

            title = "Status of DOMs of {0}".format( det )

            plot_status_DET( dominfo_dic,
                             runlist,
                             outplot_name,
                             badruns = run_dic['badruns'],
                             det_id  = det,
                             title   = title,
                             accent  = True,
                             force_recreate = force_recreate,
                             verbose = self.verbose)


#            PLOT_DIR = "plots/{0}/".format( det )
#            if not os.path.isdir( PLOT_DIR ):
#                make_plot_dirs( PLOT_DIR )
#            elif force_recreate:
#                for f in glob.glob( PLOT_DIR + "*status*.p*" ): os.remove( f )



import sys, pathlib
from pkg_resources import get_distribution, DistributionNotFound

try:
    version = get_distribution(__name__).version
except DistributionNotFound:
    version = "unknown version"

from .crawler        import crawler
from .status_plotter import status_plotter
from .scp_plotter    import scp_plotter
#from .web_maker      import web_maker
from .SC_PARAMETERS  import SC_PARAMETERS



import os, ROOT, aa, argparse
import shutil 
from SC_PARAMETERS import SC_PARAMETER_DIC as SCPD


class webpage_maker:

    def __init__.py(self):
    self.DAU_DIR   = os.environ["DOM_AUTOPSY_DIR"]
    self.PLOTS_DIR = DAU_DIR + "/plots"
    self.WEB_DIR   = DAU_DIR + "/dom_autopsy_web"

def publish( server = "jseneca@login.nikhef.nl:~/public_html" ):
    # you can now copy the webdir to a webserver -- for example
    cmd = ''
    if force_recreate:
        cmd = "scp -r {0} {1}".format( self.WEB_DIR, server )
    else:
        cmd = "scp -r {0} {1}".format( self.WEB_DIR + "/KM3NeT*", server + "/dom_autopsy_web" )

    os.system(cmd)


        
def clean():
    if not os.path.isdir( self.WEB_DIR ):
        print("ERROR: Directory {} not found.".format( self.WEB_DIR ) )
        raise ValueError
    else:
        shutil.rmtree( self.WEB_DIR )

def make( ):

    # The following puts root in batch mode; you will not see graphics
    # output on your screen.
    ROOT.gROOT.SetBatch( True )

    # We will use the WebFile object;
    # no need, compile, you can just include the .hh file...
    aa.include( os.environ['AADIR']+"/ana/histogrammer/WebFile.hh")


    if not os.path.isdir( self.WEB_DIR ):
        os.system('mkdir ' + self.WEB_DIR)

    W = ROOT.WebFile( self.WEB_DIR )

    W.write("<h1> DOM Autopsy <h1> <br> ")

    W.write("<h3> Folder structure: DET_IT > RUN_N > SC_PARAMETER <h3> <br> ")

    IMG_DIR = self.WEB_DIR + '/' + 'images'

    if not os.path.isdir( IMG_DIR ):
        os.mkdir( IMG_DIR )

    for det_obj in sorted( os.listdir( self.PLOTS_DIR ) ):

        DET_DIR     = self.PLOTS_DIR + '/' + det_obj
        WEB_DET_DIR = self.WEB_DIR  + '/' + det_obj
        if not os.path.isdir( DET_DIR ):
            continue
        if not os.path.isdir( WEB_DET_DIR ):
            os.mkdir( WEB_DET_DIR )

        W.write("<a href=\"{0}\">{1}</a> ".format(det_obj + "/" + det_obj + ".html", det_obj) )
        n_status_plots = 0
        n_run_folders  = 0

        Wdet = ROOT.WebFile( WEB_DET_DIR, det_obj + ".html")
        Wdet.write("<h1> Runs for detector {0} <h1> <br> ".format( det_obj ) )
        Wdet.write("<h3> LOOK UNDER AND RIGHT (I'm not good with html) <h3> <br> " )

        for run_obj in sorted( os.listdir( DET_DIR ), reverse = True ):
            RUN_DIR     = DET_DIR + '/' + run_obj
            WEB_RUN_DIR = WEB_DET_DIR + '/' + run_obj

            if os.path.isdir( RUN_DIR ):
                if not os.path.isdir( WEB_RUN_DIR ):
                    n_run_folders += 1
                    os.mkdir( WEB_RUN_DIR )

                Wdet.write("<a href=\"{0}\">{1}</a> <br>".format(run_obj + "/" + run_obj + ".html", run_obj) )
                n_param_folders = 0

                Wrun = ROOT.WebFile( WEB_RUN_DIR, run_obj + ".html")

                Wrun.write("<h1> Slow Control parameters for (up to) run {0} <h1> <br> ".format( run_obj ) )

                Wpars = {}
                for par_obj in sorted( os.listdir( RUN_DIR ) ):

                    print("parobj: ", par_obj )
                    param = par_obj
                    param = param.replace('SCP__', '')
                    param = param.replace('SCP_', '')
                    param = param.replace('normlog_', '')
                    param = param.replace('norm_', '')
                    param = param.replace('log_', '')
                    param = param.replace('_D0', '_D_')
                    param = param[:param.find("_D_")]

                    PAR_DIR     =     RUN_DIR + '/' + param
                    WEB_PAR_DIR = WEB_RUN_DIR + '/' + param
                    print( "param:", param )
                    if not os.path.isdir( WEB_PAR_DIR ):
                        n_param_folders += 1
                        os.mkdir( WEB_PAR_DIR )

                        Wrun.write("<a href=\"{0}\">{1}</a> <br>".format(param + "/" + param + ".html", param) )
                        if par_obj[-4:-2] == '.p':
                            Wrun.write("{0} ({1})<br>".format( SCPD[param][1], SCPD[param][0] ) )

                        Wpars[param] = ROOT.WebFile( WEB_PAR_DIR, param + ".html")

                    if par_obj[-4:] == '.png':
                        print( "parobj inside:", par_obj )
                        Wpars[param].write_img_tag( '../../../images/' + par_obj, par_obj )
                        shutil.copyfile( RUN_DIR + '/' + par_obj, IMG_DIR + '/' + par_obj )
                        print( "Wrote {0}".format( IMG_DIR + '/' + par_obj ) )

                del Wpars
                #Wrun.write("<div>{0} plots</div> <br>".format( n_plots ) )
                del Wrun
                Wdet.write("<div>{0} parameter folders.</div> <br>".format( n_param_folders ) )

            elif run_obj[-4:] == '.png':
                n_status_plots += 1
                Wdet.write_img_tag( '../images/' + run_obj, run_obj )
                shutil.copyfile( DET_DIR + '/' + run_obj, IMG_DIR + '/' + run_obj )
                print( "Wrote {0}".format( IMG_DIR + '/' + run_obj ) )
                continue
            del Wdet
            W.write("<div>{0} status plots, {1} run folders.</div> <br>".format( n_status_plots, n_run_folders ) )


    del W # destructor closes the file



import os

from unittest import TestCase

from domauto.crawler import crawler
from domauto.scp_plotter import scp_plotter
from domauto.status_plotter import status_plotter

__author__ = "Jordan Seneca"
__credits__ = []
__license__ = "MIT"
__maintainer__ = "..."
__email__ = "jseneca@km3net.de"

class TestClasses( TestCase ):

    def test_crawler( self ):

        OUTDOMS = 'DOMINFO_DIC_TEST.pickle'
        OUTRUNS = 'RUNINFO_DIC_TEST.pickle'
    
        mycrawler = crawler()
        assert 1
        #assert 0 == mycrawler.crawl( 'ORCA',
         #                            OUTRUNS,
          #                           OUTDOMS )

    def test_scp_plotter( self ):
    
        myscp_plotter = scp_plotter()
        assert 1

    def test_status_plotter( self ):
    
        mystatus_plotter = status_plotter()
        assert 1



import os
import glob
FILES_DIR = "files"

def clean_files( detector = '' ):

    if not os.path.isdir( FILES_DIR ):
        os.mkdir( FILES_DIR )
    else:
        for fp in glob.glob( FILES_DIR + '*_INFO_*' + detector + '.pickle' ):
            os.remove( fp )
            print( "Removed {}".format( fp ) )



import os, uproot
import km3pipe
import numpy as np
import matplotlib.dates as md

from .classlib import *

def make_plot_dirs( OUT_DIR ):
    print( OUT_DIR)
    RUN_DIR = OUT_DIR.split("/")[-1]
    DET_DIR = OUT_DIR[ :-len( RUN_DIR ) ] if RUN_DIR else OUT_DIR
    print(DET_DIR)
    if not os.path.isdir( DET_DIR ):
        os.mkdir( DET_DIR )
        print("Created directory {0}".format( DET_DIR ) )

    if len(OUT_DIR.split("/")[-1]) and not os.path.isdir( OUT_DIR ):
        os.mkdir( OUT_DIR )
        print("Created directory {0}".format( OUT_DIR ) )



def getruns( doms_dic ) :
    "sorted list of all runs with at least 1 live dom"
    
    r = set()
    for ( du, floor ), dominf in doms_dic.items() :
        r.update( dominf.live_runs )
        
    return sorted( r, key = lambda obj: obj.get_index() )



def get_detectors( detector_string,
                   sds ):

    if detector_string and detector_string[1:] == 'RCA':
        #Get all xRCA detectors
        detectors = [ OID_to_DETID( det_name, sds ) for det_name in sds.detectors().OID.values if detector_string in det_name ]
    elif detector_string and detector_string[1:] != 'RCA':
        if detector_string in sds.detectors().OID.values:
            #Return converted to DETID
            try:
                detector_string = OID_to_DETID( detector_string, sds )
            except:
                print( "ERROR: detector {0} not recognized...".format( detector_string ) )
                raise ValueError
        #Return DETID
        detectors = [ detector_string ]
    else:
        #Get all detectors
        detectors = [ OID_to_DETID( det_name, sds ) for det_name in sds.detectors().OID.values if 'RCA' in det_name ]
        
    return detectors
                
def process_jra_file( filename,
                      det_id,
                      runs_dic,
                      doms_dic,
                      verbose = 0) :

    try:
        run = Run( det_id, int( filename.split("_")[-2] ) )
        if verbose >= 2: print( "processing {0}, run = {1}".format( filename, run.n ) )

        f = uproot.open( filename )
        hrs = f["Detector/h_rate_summary"]
    except ValueError:
        if verbose >= 1: print( "BAD FILE: {0}, ABORTING.".format( filename ) )
        return

    except KeyError:
        if verbose >= 1: print( "NO h_rate_summary IN {0}, ABORTING.".format( filename ) )
        run.add_note(" h_rate_summary not found in {0}".format( filename ) )
        runs_dic['badruns'].append( run )
        return
    #elif hrs.GetNbinsY() < 18:
    if hrs.ynumbins < 18:
        if verbose >= 1: print( "BAD HISTOGRAM IN {0}, ABORTING.".format( filename ) )
        run.add_note(" bad h_rate_summary in {0}".format( filename ) )
        runs_dic['badruns'].append( run )
        return
    else:
        runs_dic['goodruns'].append( run )
        
    for xbin in range( hrs.xnumbins ):
        for ybin in range( hrs.ynumbins ):
            du    = int( hrs.bins[0][xbin].mean() + 1e-10 )
            floor = int( hrs.bins[1][ybin].mean() + 1e-10 )
            rate  = hrs.values[xbin][ybin]
            
            if (du,floor) not in doms_dic :
                doms_dic[ (du,floor) ] = DomInfo( du, floor )
                
            if rate == 0 : doms_dic[ (du,floor) ].dead_runs.append( run )
            if rate >  0 : doms_dic[ (du,floor) ].live_runs.append( run )

def pront( doms_dic ):

    for r in getruns( doms_dic ) :
        
        s = r.string()

        for (du,floor) in sorted(doms_dic.keys()) :

            if floor == 1 : s+= " | "
            s += '+' if r in doms_dic[(du,floor)].live_runs else '.'

        print( s )


def get_DU_dic( clb_map, det_id, verbose = False ):
    DU_dic = {}
    for DU in range( 200 ):
        for DOM in range( 20 ):
            dom = db_manager.doms.via_omkey( (DU, DOM), det_id )
            if dom == None: continue
            if verbose: print( dom )
            keys = dom.__str__().split("-")
            try:
                DU_dic[keys[0]].append( keys[1] )
            except KeyError:
                DU_dic[keys[0]] = [keys[1]]
    return DU_dic

def get_n_DU_DOM( DU_dic ):
    n_DU = len( DU_dic )
    n_DOM = 0
    for DU in DU_dic:
        if len(DU_dic[DU]) > n_DOM:
            n_DOM = len(DU_dic[DU])
    return (n_DU, n_DOM)

def get_unit( sc_param_name, sds = km3pipe.db.StreamDS() ):
    #very slow for some reason...
    params = sds.allparams().MAPNAME[ sds.allparams().loc[:, 'MAPNAME'] == sc_param_name ]
    return sds.allparams().UNIT_NAME[  params.index[0] ]

def get_description( sc_param_name, sds = km3pipe.db.StreamDS() ):
    #very slow for some reason...
    params = sds.allparams().MAPNAME[ sds.allparams().loc[:, 'MAPNAME'] == sc_param_name ]
    return sds.allparams().DESCRIPTION[  params.index[0] ]

def get_meaning( sc_param_name, sds = km3pipe.db.StreamDS() ):
    return get_description( sc_param_name, sds = km3pipe.db.StreamDS() )



def get_DOM_datagroup( scp_dataframe,
                       clb,
                       mpl_dates = True,
                       verbose = False):
    
    scp_datagroup = scp_dataframe.loc[ scp_dataframe.loc[:, 'SOURCE_NAME'] == clb.upi, :].copy()
    if not scp_datagroup.size:
        if verbose: print(" - DOM {0} empty! Skipping...".format( clb.floor ), end='')
        raise ValueError
    
    #Warning is due to the following line.
    if mpl_dates: #convert ms to s, then to matplotlib date number
        scp_datagroup.loc[:, 'UNIXTIME'] = md.epoch2num( scp_datagroup.loc[ :, 'UNIXTIME'] / 1000. )
        
    return scp_datagroup

def get_DU_datagroup( scp_dataframe,
                      DUs,
                      det_oid,
                      mpl_dates = True,
                      verbose = False):

    DU_clbs = get_DU_clbs( get_DU_omkeys( DUs, km3pipe.db.CLBMap(det_oid).omkeys, ignore_base = True) )
    scp_datagroup = scp_dataframe.loc[ scp_dataframe.loc[:, 'SOURCE_NAME'].isin( DU_clbs ), : ].copy()
    #Warning is due to the following line.
    if mpl_dates: #convert ms to s, then to matplotlib date number
        scp_datagroup.loc[:, 'UNIXTIME'] = md.epoch2num( scp_datagroup.loc[ :, 'UNIXTIME'] / 1000.)#convert ms to s, then to matplotlib date number

    return scp_datagroup

def get_DOMID_from_CLB( CLB_UPI, clbm):
    for (DOM_ID, clb) in clbm.dom_ids.items():
        if clb == CLB_UPI:
            return DOM_ID
        
def get_omkey_from_CLB( CLB_UPI, clbm):
    for (omkey, clb) in clbm.omkeys.items():
        if clb.upi in CLB_UPI:
            return omkey

def get_DUs( omkeys ):
    DUs = []
    for omkey in omkeys:
        if omkey[0] not in DUs:
            DUs.append(omkey[0])
    return DUs

def get_live_DUs( scp_dataframe, clbm):
    return set( [ get_omkey_from_CLB( upi, clbm )[0] for upi in scp_dataframe.loc[:, 'SOURCE_NAME'].unique() ] )

def get_DU_omkeys( DUs, omkeys, ignore_base = True):
    omkeys_out = {}
    for (omkey, clb) in omkeys.items():
        if omkey[0] in DUs:
            if ignore_base == False or omkey[1] != 0:
                omkeys_out[omkey] = clb
    return omkeys_out

def get_DU_clbs( DU_omkeys ):
    return np.array([(lambda oc: oc[1].upi)(oc) for oc in sorted(DU_omkeys.items())])                         

def get_DU_clbs_floor( DU_omkeys ): 
    return np.array([(lambda oc: oc[1].upi + " ({0})".format(oc[1].floor))(oc) for oc in sorted( DU_omkeys.items())])

def DETID_to_OID( detid, sds = None):
    experiment, detid_n = detid.split("_")
    serial_n = int( detid_n )
    if not sds:
        sds = km3pipe.db.StreamDS()
    dets = sds.detectors()
    return dets.loc[ dets.SERIALNUMBER == serial_n, 'OID' ].values[0]

def OID_to_DETID( oid, sds = None):
    if not sds:
        sds = km3pipe.db.StreamDS()
    dets = sds.detectors()
    return "KM3NeT_{:08d}".format( dets.loc[ dets.OID == oid, 'SERIALNUMBER' ].values[0] )



#!/usr/bin/env python3

"""
DomAuto.

Usage:
     domauto auto        [ -o=<outpath>  -d=<detector>                               -t=<threshold> -v=<v> -F ]
     domauto crawl       [               -d=<detector> -f=<doms_file> -r=<runs_file> -t=<threshold> -v=<v> -F ]
     domauto plot-status [ -o=<outpath>  -d=<detector> -f=<doms_file> -r=<runs_file>                -v=<v> -F ]
     domauto plot-scp    [ -o=<outpath>  -d=<detector>                -r=<runs_file> -p=<parameter> -v=<v> -F ]
     domauto webpage     [               -d=<detector>                                              -v=<v> -F ]
     domauto plot-scp <start> [ <end> -o=<outpath> -d=<detector>                     -p=<parameter> -v=<v> -F ]


-f --doms_file=<doms_file>        Pickle dom info file
-r --runs_file=<runs_file>        Pickle run info file
-d --detector=<detector>          Detector code, ORCA, or ARCA [default: ORCA].
-o --outpath=<outpath>            Output path [default: ]
-p --parameter=<parameter>        Will use all hits on parameter keyword, or use short, long, all for whole lists [default: short].
-F --force                        Force recreate
-t --threshold=<threshold>        Set overall threshold in # of runs, or death, coma, passout separated by spaces [default: 10 30 1].
-v --verbose=<v>                  Verbose [default: 0]
-h --help                         Display this screen
"""        
    
def main():

    import docopt
    args = docopt.docopt( __doc__ )
    verbose = int( args['--verbose'] )
    if verbose >= 2: print( "args: ", args )

    import pickle, os
    from .crawler        import crawler
    from .status_plotter import status_plotter
    from .scp_plotter    import scp_plotter
    from .classlib       import Run
    from .iolib          import get_detectors

    if not args['<start>']:
        OUTDOMS = args['--doms_file'] if args['--doms_file'] else 'DOMINFO_DIC_{0}.pickle'.format( args['--detector'] )
        OUTRUNS = args['--runs_file'] if args['--runs_file'] else 'RUNINFO_DIC_{0}.pickle'.format( args['--detector'] )
    
        print( "Added for detector {} outdoms file {}, and outruns file {}.".format( args['--detector'],
                                                                                     OUTDOMS,
                                                                                     OUTRUNS ) )

    thresholds = args['--threshold'].split(' ') if ' ' in args['--threshold'] else [ args['--threshold'] ] * 3

    #################    

    if args['crawl'] or args['auto']:
        

        #if args['--force']:
        #    clean_files( args['--detector'].split(' ') )
        #elif os.path.isfile( OUTRUNS ) and os.path.isfile( OUTDOMS ):
        #    print( "Files for {0} already exist, continuing.".format( args['--detector'] ) )
        #    if not args['auto']: exit()

        mycrawler = crawler( verbose = verbose )

        mycrawler.crawl( args['--detector'],
                         OUTRUNS,
                         OUTDOMS,
                         death_threshold   = int( thresholds[0] ),
                         coma_threshold    = int( thresholds[1] ),
                         passout_threshold = int( thresholds[2] ) )

        mycrawler.pront()





    #######################

    if args['plot-status'] or args['auto']:

        myplotter = status_plotter( verbose = verbose )

        if not os.path.isfile( OUTRUNS ):
            print( "{0} not found, exiting.".format( OUTRUNS ) )
            exit()

        if not os.path.isfile( OUTDOMS ):
            print( "{0} not found,exiting.".format( OUTDOMS ) )
            exit()
            
        myplotter.plot( args['--detector'],
                        OUTDOMS,
                        OUTRUNS,
                        args['--outpath'],
                        args['--force'] )
        
        print( "Status plotting over." )






    ##########################

    if args['plot-scp'] or args['auto']:

        if args['<start>']:
            import km3pipe
            sds = km3pipe.db.StreamDS()
            
            if 'RCA' in args['--detector']:
                if args['--detector'][1:] == 'RCA' and verbose >= 1: print("WARNING: you did not select a detector configuration. Will print run number for all detectors.")
                detectors = get_detectors( args['--detector'], sds )
            else:
                detectors = [ args['--detector'] ] 

            print( detectors )
            start = int( args['<start>'] )
            end   = int( args['<end>'] ) if args['<end>'] else start
            RUN_LIST =  [ ( Run( det, start ), [] ) for det in detectors]
            RUN_RANGE = [ 0, end - start ]

        #----------------------------------

        else:

            if not os.path.isfile( OUTRUNS ):
                print( "{0} not found, exiting.".format( OUTRUNS ) )
                exit()
                
            RUN_DIC = pickle.load( open( OUTRUNS, 'rb' ) )
            
            RUN_LIST = []
            RUN_LIST.extend( RUN_DIC['deathruns'].items() )
            RUN_LIST.extend( RUN_DIC['comaruns'].items() )
            RUN_RANGE = [5, 2]
            #RUN_APPENDIX = [ ( dacl.Run( det_id = MAN_RUN_LIST[0], n = int(run_n) ), [(-1, -1)] ) for run_n in MAN_RUN_LIST[1:] ] if MAN_RUN_LIST else []
            #RUN_LIST.extend( RUN_APPENDIX )
        
        KEYWORD = args['--parameter'].upper()
        if KEYWORD == 'ALL': KEYWORD = 'LONG'
    
        myplotter = scp_plotter( verbose = verbose )

        myplotter.batch_plot( RUN_LIST,
                              OUTPATH = args['--outpath'],
                              SCP_KEYWORD = KEYWORD,
                              RUN_RANGE = RUN_RANGE,
                              REWRITE   = args['--force'])





    ###################

    if args['webpage'] or args['auto']:
        if args['--force']:
            web_maker.clean()


        self.OUTDIR = "{0}/RUN{1}".format( self.PLOTS_DIR + "/" + self.detector, MAXRUN ) 
        make_plot_dirs( self.OUTDIR )


        if len( glob.glob( self.OUTDIR + "/*{0}*".format( PARAM_NAME) ) ) >= 5 and not args['--force']:
            print( "Directory exist and has been filled. Aborting." )
            raise ValueError


        DET_DIR = PLOTS_DIR + "/" + run.det_id 
        outfile = open( DET_DIR + "/runs_notes.md", 'a')
        for note in run.notes:
            outfile.write("RUN{0}:".format( run.n ) + note + "\n" )
        outfile.close()
        OUTDIR = "{0}/RUN{1}".format( DET_DIR, run.n ) 
        make_plot_dirs( OUTDIR )
        run.dump_notes( OUTDIR )





if __name__ == '__main__':
    main()


            #        PLOTS_DIR = './plots'
            #        if not os.path.isdir( PLOTS_DIR ):
            #            print("Plots directory not found.")
            #            raise ValueError



import numpy as np


SC_PARAMETERS = {

    'TEST': [
        ['Humid', '%', 'Humidity'],
        ['PWR_MEAS[0]', 'A', '12 Volt current, in milliamps'],
        ['TEMP', 'deg C', 'Temperature'],
        ['sys_temp_hum[2]', '%', 'CLB humidity (SHT21 chip)'],
        ['bps_v375', 'ADC', 'Voltage of 375V line (0 to 1023 ADC raw)']
    ],

    'SHORT': [
        #['AHRS_A', np.nan, np.nan], #absent?
        #['AHRS_G', np.nan, np.nan], #absent?
        #['AHRS_H', np.nan, np.nan], #absent?
        ['AHRS_PITCH', np.nan, 'AHRS Pitch'],
        ['AHRS_ROLL', np.nan, 'AHRS Roll'],
        ['AHRS_YAW', np.nan, 'AHRS Yaw'],
        ['ACDC_CUR', np.nan, 'ACDC Current, between 0 and 2560A'],
        ['DOM_Humidity', '%', 'Relative humidity'],
        ['EDFA_OOP', 'dBm', 'Output Optical Power'],
        ['Humid', '%', 'Humidity'],
        ['PWR_MEAS[0]', 'A', '12 Volt current, in milliamps'],
        ['PWR_MEAS[2]', 'A', '1.8 Volt current, in milliamps'],
        ['PWR_MEAS[1]', 'A', '1 Volt current, in milliamps'],
        ['PWR_MEAS[3]', 'A', '2.5 Volt current, in milliamps'],
        ['PWR_MEAS[6]', 'A', '3.3 Volt PMT current, in milliamps'],
        ['PWR_MEAS[4]', 'A', '3.3 Volt current, in milliamps'],
        ['PWR_MEAS[5]', 'A', '5 Volt current, in milliamps'],
        ['PWR_MEAS[10]', '?C', 'Temperature sensor voltage, in millidegree Celsius'],
        ['TEMP', 'deg C', 'Temperature'],
        ['sys_temp_hum[0]', '?C', 'FPGA temperature'],
        ['sys_temp_hum[1]', '?C', 'CLB temperature (SHT21 chip)'],
        ['sys_temp_hum[2]', '%', 'CLB humidity (SHT21 chip)'],
        ['sys_temp_hum[3]', '?C', 'Powerboard ADC temperature'],
        ['sys_temp_hum[4]', '?C', 'White Rabbit temperature'],
        ['sys_temp_hum[5]', '?C', 'SFP temperature'],
        ['acdc_cur', 'A', 'ACDC Current, between 0 and 2560A'],
        ['acdc_temp', 'C', 'ACDC Temperature in degrees Centigrade'],
        ['bps_i12', 'ADC', 'Current of 12V line (0 to 1023 ADC raw)'],
        ['bps_i375', 'ADC', 'Current of 375V line (0 to 1023 ADC raw)'],
        ['bps_iback', 'ADC', 'Backbone current (0 to 1023 ADC raw)'],
        ['edfa_oop', 'dBm', 'Output Optical Power (in milli dBm)'],
        ['edfa_pumpdc', 'A', 'Doide Pump Current (in micro A)'],
        ['edfa_setp_rd', 'dBm', 'Monitored Power Setpoint for APC mode (in milli dBm)'],
        ['edfa_temp', 'C', 'Temperature (in milli degrees celsius)'],
        ['tx_output_power', 'W', 'SFP transmitter power ']
    ],

    'LONG': [ #['#DMOutLogSecsPrescale', 's', 'Divider for seconds from beginning of day to log output variables'],
        ['ACDC_CUR', np.nan, 'ACDC Current, between 0 and 2560A'],
        ['ACDC_TEMP', '*C', 'ACDC Temperature'],
        ['ACDC_VOLT', np.nan, 'ACDC Voltage, between 300 and 556'],
        ['AHRS_A', np.nan, np.nan],
        #['SYS_DISABLE{0:1}', '1', 'Boolean value to disable AHRS'],
        ['AHRS_G', np.nan, np.nan],
        ['AHRS_H', np.nan, np.nan],
        ['AHRS_PITCH', np.nan, np.nan],
        ['AHRS_ROLL', np.nan, np.nan],
        ['AHRS_YAW', np.nan, np.nan],
        #['SYS_RUN_ENA{1:1}', np.nan, 'Boolean value to enable Acoustics (requires all SYS_RUN_ENA parameters)'],
        #['BPS_ALARM', np.nan, 'Alarm Status'],
        #['BPS_ALARM{1:1}', np.nan, 'Alarm status 12V'],
        #['BPS_ALARM{0:1}', np.nan, 'Alarm status 375V'],
        #['BPS_BREAKER', np.nan, 'Breaker status'],
        #['BPS_BREAKER{1:1}', np.nan, 'Breaker status 12V'],
        #['BPS_BREAKER{0:1}', np.nan, 'Breaker status 375V'],
        ['BPS_I12', np.nan, 'Current of 12V line (0 to 1023 ADC raw)'],
        ['(BPS_I12)', 'A', 'Current of 12V line'],
        ['BPS_I375', np.nan, 'Current of 375V line (0 to 1023 ADC raw)'],
        ['(BPS_I375)', 'A', 'Current of 375V line'],
        ['BPS_I5', np.nan, 'Current of 5V line (0 to 1023 ADC raw)'],
        ['(BPS_I5)', 'A', 'Current of 5V line'],
        ['BPS_IBACK', np.nan, 'Backbone current (0 to 1023 ADC raw)'],
        ['(BPS_IBACK)', 'A', 'Backbone current'],
        ['BPS_V375', np.nan, 'Voltage of 375V line (0 to 1023 ADC raw)'],
        ['(BPS_V375)', 'V', 'Voltage of 375V line'],
        ['CABLE_RTT', 's', 'Cable roundtrip time in ps'],
        #['CHAN_ENABLE', '1', 'Bitmask to enable PMTs'],
        ['CLBControlTimeoutMS', np.nan, 'Interval time for CLB data polling in milliseconds'],
        ['CLB_Temperature', 'C', 'Temperature of the CLB'],
        #['DAQ_ADF_DBG_path', np.nan, 'Output path for debug files'],
        #['DAQ_ADF_DOM_configuration', np.nan, 'Acoustic Data Filter search info for a DOM: DOMID Nwaveforms [IDwaveform threshold]'],
        #['DAQ_ADF_TOA_path', np.nan, 'Output path for TOA files'],
        #['DAQ_ADF_analysis_window_overlap', np.nan, 'Overlap of two analysis windows in units of samples'],
        #['DAQ_ADF_analysis_window_size', np.nan, 'Length of analysis window in units of samples'],
        #['DAQ_ADF_waveform', np.nan, 'Waveform identifier'],
        #['DAQ_ADF_wisdom_cache_path', np.nan, 'Storage path for ADF wisdom information'],
        #['DAQ_ProcessList', np.nan, 'Expects the comma-separated list of process types to be listed ("datafilters","dataqueues","datawriters","dispatchers")'],
        #['DAQ_Process_Name', np.nan, 'Name of a process'],
        #['DAQ_acou_ports', np.nan, 'Port for acoustic data'],
        #['DAQ_acou_recipient', np.nan, 'Receiver of acoustic data'],
        #['DAQ_bufferSize', np.nan, 'Size of the datafilter buffer'],
        #['DAQ_dataWriter', np.nan, 'Datawriter to output DataFilter data'],
        #['DAQ_detector', np.nan, 'The detector description in .detx format'],
        #['DAQ_dump_file_postfix', np.nan, 'Postfix of the dump file'],
        #['DAQ_dump_file_prefix', np.nan, 'Prefix of the dump file'],
        #['DAQ_logger_s', 's', 'Update period for error messages'],
        #['DAQ_maxHitsFrame', np.nan, 'Maximum number of hits in a frame'],
        #['DAQ_max_dump_size', np.nan, 'Maximum size of information to dump'],
        #['DAQ_numberOfFramesPerSlice', np.nan, 'Number of frames per slice'],
        #['DAQ_opto_ports', np.nan, 'Port for optical data'],
        #['DAQ_opto_recipients', np.nan, 'Receiver of optical data'],
        #['DAQ_out_server_address', np.nan, 'Output server address for MonRouter'],
        #['DAQ_out_server_port', np.nan, 'Output server port for MonRouter'],
        #['DAQ_path', np.nan, 'Path where the datawriter should write the data'],
        #['DAQ_pmtIdCheck', np.nan, 'Enables checking PMT Id when frames are received'],
        #['DAQ_queueDepth', np.nan, 'Maximal queue depth'],
        #['DAQ_queueSize', np.nan, 'Maximal queue size'],
        #['DAQ_run_start_time', 's', 'Unix time of run start'],
        #['DAQ_timeslice_duration', 's', 'Duration of a timeslice'],
        #['DAQ_triggerParameters=L2.TMaxLocal_ns', 's', 'Maximum local time for a L2 coincidence'],
        #['DAQ_triggerParameters=L2.ctMin', np.nan, 'CTMin of L2 coincidence'],
        #['DAQ_triggerParameters=L2.numberOfHits', np.nan, 'Number of hits for a L2 coincidence'],
        #['DAQ_triggerParameters=L2Min', np.nan, 'Minimum number of L2 triggers'],
        #['DAQ_triggerParameters=SN.TMaxLocal_ns', 's', 'Maximum local time for a SN coincidence'],
        #['DAQ_triggerParameters=SN.ctMin', np.nan, 'CTMin of SN coincidence'],
        #['DAQ_triggerParameters=SN.numberOfHits', np.nan, 'Number of hits for a SN coincidence'],
        #['DAQ_triggerParameters=TMaxEvent_ns', 's', 'Maximal time between first and last hit'],
        #['DAQ_triggerParameters=TMaxLocal_ns', 's', 'Maximum local time'],
        #['DAQ_triggerParameters=combineL1', np.nan, 'Combine consequtive L1s to one'],
        #['DAQ_triggerParameters=ctMin', np.nan, 'CTMin of triggers'],
        #['DAQ_triggerParameters=highRateVeto_Hz', 'Hz', 'Limit on rate'],
        #['DAQ_triggerParameters=numberOfBins', np.nan, 'Number of bins'],
        #['DAQ_triggerParameters=trigger3DMuon.DMax_m', 'm', 'Maximum distance of a trigger from muon'],
        #['DAQ_triggerParameters=trigger3DMuon.TMaxExtra_ns', 's', 'Maximum extra time for a muon trigger'],
        #['DAQ_triggerParameters=trigger3DMuon.enabled', np.nan, 'Enables/disables the muon trigger'],
        #['DAQ_triggerParameters=trigger3DMuon.factoryLimit', np.nan, 'Maximum number of muon triggers'],
        #['DAQ_triggerParameters=trigger3DMuon.gridAngle_deg', 'deg', 'Grid angle for muons in degrees'],
        #['DAQ_triggerParameters=trigger3DMuon.numberOfHits', np.nan, 'Number of hits for a muon trigger'],
        #['DAQ_triggerParameters=trigger3DMuon.numberOfModules', np.nan, 'Number of modules for a muon trigger'],
        #['DAQ_triggerParameters=trigger3DMuon.roadWidth_m', 'm', 'Road width for track reconstruction'],
        #['DAQ_triggerParameters=trigger3DShower.DMax_m', 'm', 'Maximum distance of a trigger from a 3D shower'],
        #['DAQ_triggerParameters=trigger3DShower.TMaxExtra_ns', 's', 'Maximum extra time for a shower trigger'],
        #['DAQ_triggerParameters=trigger3DShower.enabled', np.nan, 'Enables/disables the shower trigger'],
        #['DAQ_triggerParameters=trigger3DShower.factoryLimit', np.nan, 'Maximum number of showers triggers'],
        #['DAQ_triggerParameters=trigger3DShower.numberOfHits', np.nan, 'Number of hits for 3D shower trigger'],
        #['DAQ_triggerParameters=trigger3DShower.numberOfModules', np.nan, 'Number of moduels for a shower trigger'],
        #['DAQ_triggerParameters=triggerMXShower.DMax_m', 'm', 'Maximum distance of a trigger from an MX trigger'],
        #['DAQ_triggerParameters=triggerMXShower.TMaxExtra_ns', 's', 'Maximum extra time for an MX trigger'],
        #['DAQ_triggerParameters=triggerMXShower.enabled', np.nan, 'Enables/disables the MX trigger'],
        #['DAQ_triggerParameters=triggerMXShower.factoryLimit', np.nan, 'Maximum number of MX triggers'],
        #['DAQ_triggerParameters=triggerMXShower.numberOfHits', np.nan, 'Number of hits for MX trigger'],
        #['DAQ_triggerParameters=triggerMXShower.numberOfModules', np.nan, 'Number of modules necessary to trigger for MX trigger'],
        #['DAQ_triggerParameters=triggerNB.DMax_m', 'm', 'Maximum distance for the np.nanobeacon trigger'],
        #['DAQ_triggerParameters=triggerNB.enabled', np.nan, 'Enables/disables the np.nanobeacon trigger'],
        #['DAQ_triggerParameters=triggerNB.pmts', np.nan, 'List of involved PMTs for the np.nanobeacon trigger'],
        #['DAQ_triggerParameters=writeL0', np.nan, 'Write L0 triggers'],
        #['DAQ_triggerParameters=writeL0.prescale', np.nan, 'Prescale of L0 timeslices writing'],
        #['DAQ_triggerParameters=writeL1', np.nan, 'Write L1 timeslices'],
        #['DAQ_triggerParameters=writeL1.prescale', np.nan, 'Prescale of L1 timeslices writing'],
        #['DAQ_triggerParameters=writeL2', np.nan, 'Write L2 timeslices'],
        #['DAQ_triggerParameters=writeL2.prescale', np.nan, 'Prescale of L2 timeslices writing'],
        #['DAQ_triggerParameters=writeSN', np.nan, 'Write SN timeslices'],
        #['DAQ_triggerParameters=writeSN.prescale', np.nan, 'Prescale of SN timeslices writing'],
        #['DAQ_triggerParameters=writeSummary', np.nan, 'Detail level of summary'],
        #['DAQ_triggerParameters=writeSummary.prescale', np.nan, 'Prescale of summary timeslices writing'],
        #['DAQ_triggerParameters=writeTimeslices', np.nan, 'Writes full dumps of timeslices'],
        #['DAQ_triggerParameters=writeTimeslices.prescale', np.nan, 'Prescale of timeslice writing'],
        #['DAQ_update_s', 's', 'Update period for status messages'],
        ['DOM_Humidity', '%', 'Relative humidity'],
        #['DOM_ID', np.nan, 'DOM Identifier'],
        ['EDFA_EMI', np.nan, 'Emission'],
        ['EDFA_IOP', 'dBm', 'Input Optical Power'],
        ['EDFA_OOP', 'dBm', 'Output Optical Power'],
        ['EDFA_PUMPDC', 'A', 'Diode Pump Current'],
        ['EDFA_SETP_RD', 'dBm', 'Monitored Power Setpoint for APC mode'],
        #['EDFA_SETP_WR', 'dBm', 'Settings for Power Setpoint'],
        #['EDFA_SN', np.nan, 'Serial Number'],
        ['EDFA_TEMP', '?C', 'Temperature '],
        ['FPGA_Temperature', 'C', 'Temperature of the FPGA'],
        #['HW_DATE_REV', np.nan, 'Date and Revision'],
        ['Humid', '%', 'Humidity'],
        #['INST_FAIL', np.nan, 'Bitmask containing error devices'],
        #['INST_FAIL{2:1}', np.nan, 'Failure status for ACDC'],
        #['INST_FAIL{1:1}', np.nan, 'Failure status for BPS'],
        #['INST_FAIL{0:1}', np.nan, 'Failure status for EDFA'],
        #['IPMUX_MOD_IP', np.nan, 'Units IP address'],
        #['IPMUX_PORTS', np.nan, 'IP Mux Source and target port'],
        #['IPMUX_SRV_IP', np.nan, 'Target IP Address for IPMUX Packets'],
        #['SYS_RUN_ENA{2:1}', '1', 'Boolean value to enable Monitoring (requires all SYS_RUN_ENA parameters)'],
        #['NP.NANO_Delay', 's', 'Np.Nano-beacon delay in 16 ns ticks'],
        #['NP.NANO_Length', 's', 'Length of pulse in 16 ns ticks'],
        #['NP.NANO_PCount', '1', 'Np.Nano-beacon pulse train count, each second'],
        #['NP.NANO_Period', 's', 'Period of beacon in 16 ns ticks'],
        #['NP.NANO_Volt', 'V', 'Np.Nano-beacon voltage in millivolts'],
        #['SYS_RUN_ENA{3:1}', np.nan, 'Boolean value to enable Np.Nanobeacons (requires all SYS_RUN_ENA parameters)'],
        #['SYS_DISABLE{3:1}', np.nan, 'Boolean value to disable Np.Nanobeacon step up regulator'],
        #['PIEZO_ENABLE', np.nan, np.nan],
        #['PIEZO_ENABLE', np.nan, np.nan],
        #['PMT_ERR', np.nan, 'Channel error - 0 no error, 1 fault, 2 communication error'],
        #['CHAN_ENABLE(31)', np.nan, 'Bit array to enable PMTs'],
        ['PMT_HIGHVOLT', 'V', 'High Voltage'],
        #['SYS_DISABLE{2:1}', '1', 'Boolean value to disable PMT HV (power line to HV altogether)'],
        #['hr_veto_ena_ch(31)', np.nan, 'Enable high rate veto per PMT'],
        #['PMT_ID', np.nan, 'PMT Identifier of each channel'],
        #['PMT_MAPPING', np.nan, 'Maps a PMT Channel to an actual octopus module'],
        #['mulhit_ena_ch(31)', np.nan, 'Enable multiple hits per PMT'],
        #['PMT_THRESHOLD', 'V', 'PMT THreshold'],
        ['PWR_MEAS[0]', 'A', '12 Volt current, in milliamps'],
        ['PWR_MEAS[9]', 'V', '12 Volt level, in millivolts'],
        ['PWR_MEAS[2]', 'A', '1.8 Volt current, in milliamps'],
        ['PWR_MEAS[12]', 'V', '1.8 Volt level, in millivolts'],
        ['PWR_MEAS[1]', 'A', '1 Volt current, in milliamps'],
        ['PWR_MEAS[11]', 'V', '1 Volt level, in millivolts'],
        ['PWR_MEAS[3]', 'A', '2.5 Volt current, in milliamps'],
        ['PWR_MEAS[13]', 'V', '2.5 Volt level, in millivolts'],
        ['PWR_MEAS[6]', 'A', '3.3 Volt PMT current, in milliamps'],
        ['PWR_MEAS[16]', 'V', '3.3 Volt level, in millivolts'],
        ['PWR_MEAS[4]', 'A', '3.3 Volt current, in milliamps'],
        ['PWR_MEAS[14]', 'V', '3.3 Volt level, in millivolts'],
        ['PWR_MEAS[5]', 'A', '5 Volt current, in milliamps'],
        ['PWR_MEAS[15]', 'V', '5 Volt level, in millivolts'],
        ['PWR_MEAS[17]', 'V', 'DAC Control voltage, in millivolts'],
        ['PWR_MEAS[10]', '?C', 'Temperature sensor voltage, in millidegree Celsius'],
        ['PWR_MEAS[7]', 'A', 'LED current, in milliamps'],
        ['PWR_MEAS[8]', 'V', 'LED Voltage level, in millivolts'],
        #['RUN_NUMBER', np.nan, 'Run number'],
        ['SC_PORT', np.nan, 'Slow control'],
        #['SC_SRV_IP', np.nan, 'Target Ip for slow control events'],
        ['SFP_Temperature', 'C', 'Temperature of the SFP'],
        #['SYS_MAC', np.nan, 'Unit Mac Address'],
        #['SYS_RUN_ENA', '1', 'Bitmask parameter that defines CLB subsystems that can actually run.'],
        #['TDC_ENABLE', np.nan, 'TDC Enable.  Bits 0..30 Channels 1-31 Bit 31 - core enable'],
        #['TDC_ENABLE', np.nan, 'TDC Enable.  Bits 0..30 Channels 1-31 Bit 31 - core enable'],
        #['SYS_RUN_ENA{0:1}', '1', 'Boolean value to enable TDC (requires all SYS_RUN_ENA parameters)'],
        ['TEMP', 'deg C', 'Temperature'],
        ['sys_temp_hum[0]', '?C', 'FPGA temperature'],
        ['sys_temp_hum[1]', '?C', 'CLB temperature (SHT21 chip)'],
        ['sys_temp_hum[2]', '%', 'CLB humidity (SHT21 chip)'],
        ['sys_temp_hum[3]', '?C', 'Powerboard ADC temperature'],
        ['sys_temp_hum[4]', '?C', 'White Rabbit temperature'],
        ['sys_temp_hum[5]', '?C', 'SFP temperature'],
        #['SYS_DISABLE{1:1}', '1', 'Boolean value to disable Temperature/Humidity'],
        #['WR_Bitslide', 's', 'White Rabbit Bitslide value'],
        #['WR_DELTA', np.nan, 'White Rabbit Deltas'],
        #['WR_DELTA[3]', 's', 'White Rabbit Delay value Master RX'],
        #['WR_DELTA[2]', 's', 'White Rabbit Delay value Master TX'],
        #['WR_DELTA[1]', 's', 'White Rabbit Delay value Slave RX'],
        #['WR_DELTA[0]', 's', 'White Rabbit Delay value Slave TX'],
        #['WR_MU', 's', 'White Rabbit Round-Trip time'],
        ['WR_ST_GEN', np.nan, 'White Rabbit general state'],
        #['WR_ST_PTP', np.nan, 'White Rabbit PTP State'],
        #['WR_ST_SERV', np.nan, 'White Rabbit servo state'],
        ['WR_TEMP', np.nan, 'Temperature from white rabbit'],
        ['WR_TEMP', np.nan, 'Temperature from white rabbit'],
        ['acdc_cur', 'A', 'ACDC Current, between 0 and 2560A'],
        ['acdc_temp', 'C', 'ACDC Temperature in degrees Centigrade'],
        ['acdc_volt', 'V', 'ACDC Voltage, between 300 and 556'],
        #['acou_chan', np.nan, '0=Enable both acoustic channels,1=Enable only channel 1,2=Enable only channel 2'],
        #['acou_res', np.nan, '0=12 bit resolution,1=16 bit resolution,2=24 bit resolution'],
        #['bps_alarm', np.nan, 'Alarm Status'],
        #['bps_breaker', 'ADC', 'Breaker status'],
        ['bps_i12', 'ADC', 'Current of 12V line (0 to 1023 ADC raw)'],
        ['bps_i375', 'ADC', 'Current of 375V line (0 to 1023 ADC raw)'],
        ['bps_iback', 'ADC', 'Backbone current (0 to 1023 ADC raw)'],
        ['bps_v375', 'ADC', 'Voltage of 375V line (0 to 1023 ADC raw)'],
        ['bps_v5', 'ADC', 'Voltage of 5V line (0 to 1023 ADC raw)'],
        ['cable_rtt', 's', 'Cable roundtrip time in ps'],
        ['edfa_emi', np.nan, 'Emission'],
        ['edfa_iop', 'dBm', 'Input Optical Power (in milli dBm)'],
        ['edfa_oop', 'dBm', 'Output Optical Power (in milli dBm)'],
        ['edfa_pumpdc', 'A', 'Doide Pump Current (in micro A)'],
        ['edfa_setp_rd', 'dBm', 'Monitored Power Setpoint for APC mode (in milli dBm)'],
        #['edfa_setp_wr', 'dBm', 'Settings for Power Setpoint (in dBm)'],
        #['edfa_sn', np.nan, 'Serial number (Ascii string)'],
        ['edfa_temp', 'C', 'Temperature (in milli degrees celsius)'],
        #['hr_veto_ena_ch', '1', 'Bitmask to enable high rate veto on PMTs'],
        #['hr_veto_thres', np.nan, 'Threshold in hits per timeslice'],
        #['mulhit_ena_ch', '1', 'Bitmask to enable multi-hits on PMTs'],
        #['pwr_meas', np.nan, 'power measurment'],
        ['rx_input_power', 'W', 'SFP receiver input power'],
        #['stmach_pktsize', np.nan, np.nan],
        #['sys_temp_hum', np.nan, 'Mixed array of temperature and humidity (0=FPGA,1=CLB,2=CLB_hum,3=POWER,4=WR,5=SFP)'],
        #['time_slice_dur', 's', 'Timeslice duration'],
        ['tx_output_power', 'W', 'SFP transmitter power ']
    ]
}

SC_PARAMETER_DIC = {'#DMOutLogSecsPrescale': [ 's', 'Divider for seconds from beginning of day to log output variables'],
                    'ACDC_CUR': [ np.nan, 'ACDC Current, between 0 and 2560A'],
                    'ACDC_TEMP': [ '*C', 'ACDC Temperature'],
                    'ACDC_VOLT': [ np.nan, 'ACDC Voltage, between 300 and 556'],
                    'AHRS_A': [ np.nan, np.nan],
                    'SYS_DISABLE{0:1}': [ '1', 'Boolean value to disable AHRS'],
                    'AHRS_G': [ np.nan, np.nan],
                    'AHRS_H': [ np.nan, np.nan],
                    'AHRS_PITCH': [ np.nan, np.nan],
                    'AHRS_ROLL': [ np.nan, np.nan],
                    'AHRS_YAW': [ np.nan, np.nan],
                    'SYS_RUN_ENA{1:1}': [ np.nan, 'Boolean value to enable Acoustics (requires all SYS_RUN_ENA parameters)'],
                    'BPS_ALARM': [ np.nan, 'Alarm Status'],
                    'BPS_ALARM{1:1}': [ np.nan, 'Alarm status 12V'],
                    'BPS_ALARM{0:1}': [ np.nan, 'Alarm status 375V'],
                    'BPS_BREAKER': [ np.nan, 'Breaker status'],
                    'BPS_BREAKER{1:1}': [ np.nan, 'Breaker status 12V'],
                    'BPS_BREAKER{0:1}': [ np.nan, 'Breaker status 375V'],
                    'BPS_I12': [ np.nan, 'Current of 12V line (0 to 1023 ADC raw)'],
                    '(BPS_I12)': [ 'A', 'Current of 12V line'],
                    'BPS_I375': [ np.nan, 'Current of 375V line (0 to 1023 ADC raw)'],
                    '(BPS_I375)': [ 'A', 'Current of 375V line'],
                    'BPS_I5': [ np.nan, 'Current of 5V line (0 to 1023 ADC raw)'],
                    '(BPS_I5)': [ 'A', 'Current of 5V line'],
                    'BPS_IBACK': [ np.nan, 'Backbone current (0 to 1023 ADC raw)'],
                    '(BPS_IBACK)': [ 'A', 'Backbone current'],
                    'BPS_V375': [ np.nan, 'Voltage of 375V line (0 to 1023 ADC raw)'],
                    '(BPS_V375)': [ 'V', 'Voltage of 375V line'],
                    'CABLE_RTT': [ 's', 'Cable roundtrip time in ps'],
                    'CHAN_ENABLE': [ '1', 'Bitmask to enable PMTs'],
                    'CLBControlTimeoutMS': [ np.nan, 'Interval time for CLB data polling in milliseconds'],
                    'CLB_Temperature': [ 'C', 'Temperature of the CLB'],
                    'DAQ_ADF_DBG_path': [ np.nan, 'Output path for debug files'],
                    'DAQ_ADF_DOM_configuration': [ np.nan, 'Acoustic Data Filter search info for a DOM: DOMID Nwaveforms [IDwaveform threshold]'],
                    'DAQ_ADF_TOA_path': [ np.nan, 'Output path for TOA files'],
                    'DAQ_ADF_analysis_window_overlap': [ np.nan, 'Overlap of two analysis windows in units of samples'],
                    'DAQ_ADF_analysis_window_size': [ np.nan, 'Length of analysis window in units of samples'],
                    'DAQ_ADF_waveform': [ np.nan, 'Waveform identifier'],
                    'DAQ_ADF_wisdom_cache_path': [ np.nan, 'Storage path for ADF wisdom information'],
                    'DAQ_ProcessList': [ np.nan, 'Expects the comma-separated list of process types to be listed ("datafilters","dataqueues","datawriters","dispatchers")'],
                    'DAQ_Process_Name': [ np.nan, 'Name of a process'],
                    'DAQ_acou_ports': [ np.nan, 'Port for acoustic data'],
                    'DAQ_acou_recipient': [ np.nan, 'Receiver of acoustic data'],
                    'DAQ_bufferSize': [ np.nan, 'Size of the datafilter buffer'],
                    'DAQ_dataWriter': [ np.nan, 'Datawriter to output DataFilter data'],
                    'DAQ_detector': [ np.nan, 'The detector description in .detx format'],
                    'DAQ_dump_file_postfix': [ np.nan, 'Postfix of the dump file'],
                    'DAQ_dump_file_prefix': [ np.nan, 'Prefix of the dump file'],
                    'DAQ_logger_s': [ 's', 'Update period for error messages'],
                    'DAQ_maxHitsFrame': [ np.nan, 'Maximum number of hits in a frame'],
                    'DAQ_max_dump_size': [ np.nan, 'Maximum size of information to dump'],
                    'DAQ_numberOfFramesPerSlice': [ np.nan, 'Number of frames per slice'],
                    'DAQ_opto_ports': [ np.nan, 'Port for optical data'],
                    'DAQ_opto_recipients': [ np.nan, 'Receiver of optical data'],
                    'DAQ_out_server_address': [ np.nan, 'Output server address for MonRouter'],
                    'DAQ_out_server_port': [ np.nan, 'Output server port for MonRouter'],
                    'DAQ_path': [ np.nan, 'Path where the datawriter should write the data'],
                    'DAQ_pmtIdCheck': [ np.nan, 'Enables checking PMT Id when frames are received'],
                    'DAQ_queueDepth': [ np.nan, 'Maximal queue depth'],
                    'DAQ_queueSize': [ np.nan, 'Maximal queue size'],
                    'DAQ_run_start_time': [ 's', 'Unix time of run start'],
                    'DAQ_timeslice_duration': [ 's', 'Duration of a timeslice'],
                    'DAQ_triggerParameters=L2.TMaxLocal_ns': [ 's', 'Maximum local time for a L2 coincidence'],
                    'DAQ_triggerParameters=L2.ctMin': [ np.nan, 'CTMin of L2 coincidence'],
                    'DAQ_triggerParameters=L2.numberOfHits': [ np.nan, 'Number of hits for a L2 coincidence'],
                    'DAQ_triggerParameters=L2Min': [ np.nan, 'Minimum number of L2 triggers'],
                    'DAQ_triggerParameters=SN.TMaxLocal_ns': [ 's', 'Maximum local time for a SN coincidence'],
                    'DAQ_triggerParameters=SN.ctMin': [ np.nan, 'CTMin of SN coincidence'],
                    'DAQ_triggerParameters=SN.numberOfHits': [ np.nan, 'Number of hits for a SN coincidence'],
                    'DAQ_triggerParameters=TMaxEvent_ns': [ 's', 'Maximal time between first and last hit'],
                    'DAQ_triggerParameters=TMaxLocal_ns': [ 's', 'Maximum local time'],
                    'DAQ_triggerParameters=combineL1': [ np.nan, 'Combine consequtive L1s to one'],
                    'DAQ_triggerParameters=ctMin': [ np.nan, 'CTMin of triggers'],
                    'DAQ_triggerParameters=highRateVeto_Hz': [ 'Hz', 'Limit on rate'],
                    'DAQ_triggerParameters=numberOfBins': [ np.nan, 'Number of bins'],
                    'DAQ_triggerParameters=trigger3DMuon.DMax_m': [ 'm', 'Maximum distance of a trigger from muon'],
                    'DAQ_triggerParameters=trigger3DMuon.TMaxExtra_ns': [ 's', 'Maximum extra time for a muon trigger'],
                    'DAQ_triggerParameters=trigger3DMuon.enabled': [ np.nan, 'Enables/disables the muon trigger'],
                    'DAQ_triggerParameters=trigger3DMuon.factoryLimit': [ np.nan, 'Maximum number of muon triggers'],
                    'DAQ_triggerParameters=trigger3DMuon.gridAngle_deg': [ 'deg', 'Grid angle for muons in degrees'],
                    'DAQ_triggerParameters=trigger3DMuon.numberOfHits': [ np.nan, 'Number of hits for a muon trigger'],
                    'DAQ_triggerParameters=trigger3DMuon.numberOfModules': [ np.nan, 'Number of modules for a muon trigger'],
                    'DAQ_triggerParameters=trigger3DMuon.roadWidth_m': [ 'm', 'Road width for track reconstruction'],
                    'DAQ_triggerParameters=trigger3DShower.DMax_m': [ 'm', 'Maximum distance of a trigger from a 3D shower'],
                    'DAQ_triggerParameters=trigger3DShower.TMaxExtra_ns': [ 's', 'Maximum extra time for a shower trigger'],
                    'DAQ_triggerParameters=trigger3DShower.enabled': [ np.nan, 'Enables/disables the shower trigger'],
                    'DAQ_triggerParameters=trigger3DShower.factoryLimit': [ np.nan, 'Maximum number of showers triggers'],
                    'DAQ_triggerParameters=trigger3DShower.numberOfHits': [ np.nan, 'Number of hits for 3D shower trigger'],
                    'DAQ_triggerParameters=trigger3DShower.numberOfModules': [ np.nan, 'Number of moduels for a shower trigger'],
                    'DAQ_triggerParameters=triggerMXShower.DMax_m': [ 'm', 'Maximum distance of a trigger from an MX trigger'],
                    'DAQ_triggerParameters=triggerMXShower.TMaxExtra_ns': [ 's', 'Maximum extra time for an MX trigger'],
                    'DAQ_triggerParameters=triggerMXShower.enabled': [ np.nan, 'Enables/disables the MX trigger'],
                    'DAQ_triggerParameters=triggerMXShower.factoryLimit': [ np.nan, 'Maximum number of MX triggers'],
                    'DAQ_triggerParameters=triggerMXShower.numberOfHits': [ np.nan, 'Number of hits for MX trigger'],
                    'DAQ_triggerParameters=triggerMXShower.numberOfModules': [ np.nan, 'Number of modules necessary to trigger for MX trigger'],
                    'DAQ_triggerParameters=triggerNB.DMax_m': [ 'm', 'Maximum distance for the np.nanobeacon trigger'],
                    'DAQ_triggerParameters=triggerNB.enabled': [ np.nan, 'Enables/disables the np.nanobeacon trigger'],
                    'DAQ_triggerParameters=triggerNB.pmts': [ np.nan, 'List of involved PMTs for the np.nanobeacon trigger'],
                    'DAQ_triggerParameters=writeL0': [ np.nan, 'Write L0 triggers'],
                    'DAQ_triggerParameters=writeL0.prescale': [ np.nan, 'Prescale of L0 timeslices writing'],
                    'DAQ_triggerParameters=writeL1': [ np.nan, 'Write L1 timeslices'],
                    'DAQ_triggerParameters=writeL1.prescale': [ np.nan, 'Prescale of L1 timeslices writing'],
                    'DAQ_triggerParameters=writeL2': [ np.nan, 'Write L2 timeslices'],
                    'DAQ_triggerParameters=writeL2.prescale': [ np.nan, 'Prescale of L2 timeslices writing'],
                    'DAQ_triggerParameters=writeSN': [ np.nan, 'Write SN timeslices'],
                    'DAQ_triggerParameters=writeSN.prescale': [ np.nan, 'Prescale of SN timeslices writing'],
                    'DAQ_triggerParameters=writeSummary': [ np.nan, 'Detail level of summary'],
                    'DAQ_triggerParameters=writeSummary.prescale': [ np.nan, 'Prescale of summary timeslices writing'],
                    'DAQ_triggerParameters=writeTimeslices': [ np.nan, 'Writes full dumps of timeslices'],
                    'DAQ_triggerParameters=writeTimeslices.prescale': [ np.nan, 'Prescale of timeslice writing'],
                    'DAQ_update_s': [ 's', 'Update period for status messages'],
                    'DOM_Humidity': [ '%', 'Relative humidity'],
                    'DOM_ID': [ np.nan, 'DOM Identifier'],
                    'EDFA_EMI': [ np.nan, 'Emission'],
                    'EDFA_IOP': [ 'dBm', 'Input Optical Power'],
                    'EDFA_OOP': [ 'dBm', 'Output Optical Power'],
                    'EDFA_PUMPDC': [ 'A', 'Diode Pump Current'],
                    'EDFA_SETP_RD': [ 'dBm', 'Monitored Power Setpoint for APC mode'],
                    'EDFA_SETP_WR': [ 'dBm', 'Settings for Power Setpoint'],
                    'EDFA_SN': [ np.nan, 'Serial Number'],
                    'EDFA_TEMP': [ '?C', 'Temperature '],
                    'FPGA_Temperature': [ 'C', 'Temperature of the FPGA'],
                    'HW_DATE_REV': [ np.nan, 'Date and Revision'],
                    'Humid': [ '%', 'Humidity'],
                    'INST_FAIL': [ np.nan, 'Bitmask containing error devices'],
                    'INST_FAIL{2:1}': [ np.nan, 'Failure status for ACDC'],
                    'INST_FAIL{1:1}': [ np.nan, 'Failure status for BPS'],
                    'INST_FAIL{0:1}': [ np.nan, 'Failure status for EDFA'],
                    'IPMUX_MOD_IP': [ np.nan, 'Units IP address'],
                    'IPMUX_PORTS': [ np.nan, 'IP Mux Source and target port'],
                    'IPMUX_SRV_IP': [ np.nan, 'Target IP Address for IPMUX Packets'],
                    'SYS_RUN_ENA{2:1}': [ '1', 'Boolean value to enable Monitoring (requires all SYS_RUN_ENA parameters)'],
                    'NP.NANO_Delay': [ 's', 'Np.Nano-beacon delay in 16 ns ticks'],
                    'NP.NANO_Length': [ 's', 'Length of pulse in 16 ns ticks'],
                    'NP.NANO_PCount': [ '1', 'Np.Nano-beacon pulse train count, each second'],
                    'NP.NANO_Period': [ 's', 'Period of beacon in 16 ns ticks'],
                    'NP.NANO_Volt': [ 'V', 'Np.Nano-beacon voltage in millivolts'],
                    'SYS_RUN_ENA{3:1}': [ np.nan, 'Boolean value to enable Np.Nanobeacons (requires all SYS_RUN_ENA parameters)'],
                    'SYS_DISABLE{3:1}': [ np.nan, 'Boolean value to disable Np.Nanobeacon step up regulator'],
                    'PIEZO_ENABLE': [ np.nan, np.nan],
                    'PIEZO_ENABLE': [ np.nan, np.nan],
                    'PMT_ERR': [ np.nan, 'Channel error - 0 no error, 1 fault, 2 communication error'],
                    'CHAN_ENABLE(31)': [ np.nan, 'Bit array to enable PMTs'],
                    'PMT_HIGHVOLT': [ 'V', 'High Voltage'],
                    'SYS_DISABLE{2:1}': [ '1', 'Boolean value to disable PMT HV (power line to HV altogether)'],
                    'hr_veto_ena_ch(31)': [ np.nan, 'Enable high rate veto per PMT'],
                    'PMT_ID': [ np.nan, 'PMT Identifier of each channel'],
                    'PMT_MAPPING': [ np.nan, 'Maps a PMT Channel to an actual octopus module'],
                    'mulhit_ena_ch(31)': [ np.nan, 'Enable multiple hits per PMT'],
                    'PMT_THRESHOLD': [ 'V', 'PMT THreshold'],
                    'PWR_MEAS[0]': [ 'A', '12 Volt current, in milliamps'],
                    'PWR_MEAS[9]': [ 'V', '12 Volt level, in millivolts'],
                    'PWR_MEAS[2]': [ 'A', '1.8 Volt current, in milliamps'],
                    'PWR_MEAS[12]': [ 'V', '1.8 Volt level, in millivolts'],
                    'PWR_MEAS[1]': [ 'A', '1 Volt current, in milliamps'],
                    'PWR_MEAS[11]': [ 'V', '1 Volt level, in millivolts'],
                    'PWR_MEAS[3]': [ 'A', '2.5 Volt current, in milliamps'],
                    'PWR_MEAS[13]': [ 'V', '2.5 Volt level, in millivolts'],
                    'PWR_MEAS[6]': [ 'A', '3.3 Volt PMT current, in milliamps'],
                    'PWR_MEAS[16]': [ 'V', '3.3 Volt level, in millivolts'],
                    'PWR_MEAS[4]': [ 'A', '3.3 Volt current, in milliamps'],
                    'PWR_MEAS[14]': [ 'V', '3.3 Volt level, in millivolts'],
                    'PWR_MEAS[5]': [ 'A', '5 Volt current, in milliamps'],
                    'PWR_MEAS[15]': [ 'V', '5 Volt level, in millivolts'],
                    'PWR_MEAS[17]': [ 'V', 'DAC Control voltage, in millivolts'],
                    'PWR_MEAS[10]': [ '?C', 'Temperature sensor voltage, in millidegree Celsius'],
                    'PWR_MEAS[7]': [ 'A', 'LED current, in milliamps'],
                    'PWR_MEAS[8]': [ 'V', 'LED Voltage level, in millivolts'],
                    'RUN_NUMBER': [ np.nan, 'Run number'],
                    'SC_PORT': [ np.nan, 'Slow control'],
                    'SC_SRV_IP': [ np.nan, 'Target Ip for slow control events'],
                    'SFP_Temperature': [ 'C', 'Temperature of the SFP'],
                    'SYS_MAC': [ np.nan, 'Unit Mac Address'],
                    'SYS_RUN_ENA': [ '1', 'Bitmask parameter that defines CLB subsystems that can actually run.'],
                    'TDC_ENABLE': [ np.nan, 'TDC Enable.  Bits 0..30 Channels 1-31 Bit 31 - core enable'],
                    'TDC_ENABLE': [ np.nan, 'TDC Enable.  Bits 0..30 Channels 1-31 Bit 31 - core enable'],
                    'SYS_RUN_ENA{0:1}': [ '1', 'Boolean value to enable TDC (requires all SYS_RUN_ENA parameters)'],
                    'TEMP': [ 'deg C', 'Temperature'],
                    'sys_temp_hum[0]': [ '?C', 'FPGA temperature'],
                    'sys_temp_hum[1]': [ '?C', 'CLB temperature (SHT21 chip)'],
                    'sys_temp_hum[2]': [ '%', 'CLB humidity (SHT21 chip)'],
                    'sys_temp_hum[3]': [ '?C', 'Powerboard ADC temperature'],
                    'sys_temp_hum[4]': [ '?C', 'White Rabbit temperature'],
                    'sys_temp_hum[5]': [ '?C', 'SFP temperature'],
                    'SYS_DISABLE{1:1}': [ '1', 'Boolean value to disable Temperature/Humidity'],
                    'WR_Bitslide': [ 's', 'White Rabbit Bitslide value'],
                    'WR_DELTA': [ np.nan, 'White Rabbit Deltas'],
                    'WR_DELTA[3]': [ 's', 'White Rabbit Delay value Master RX'],
                    'WR_DELTA[2]': [ 's', 'White Rabbit Delay value Master TX'],
                    'WR_DELTA[1]': [ 's', 'White Rabbit Delay value Slave RX'],
                    'WR_DELTA[0]': [ 's', 'White Rabbit Delay value Slave TX'],
                    'WR_MU': [ 's', 'White Rabbit Round-Trip time'],
                    'WR_ST_GEN': [ np.nan, 'White Rabbit general state'],
                    'WR_ST_PTP': [ np.nan, 'White Rabbit PTP State'],
                    'WR_ST_SERV': [ np.nan, 'White Rabbit servo state'],
                    'WR_TEMP': [ np.nan, 'Temperature from white rabbit'],
                    'WR_TEMP': [ np.nan, 'Temperature from white rabbit'],
                    'acdc_cur': [ 'A', 'ACDC Current, between 0 and 2560A'],
                    'acdc_temp': [ 'C', 'ACDC Temperature in degrees Centigrade'],
                    'acdc_volt': [ 'V', 'ACDC Voltage, between 300 and 556'],
                    'acou_chan': [ np.nan, '0=Enable both acoustic channels,1=Enable only channel 1,2=Enable only channel 2'],
                    'acou_res': [ np.nan, '0=12 bit resolution,1=16 bit resolution,2=24 bit resolution'],
                    'bps_alarm': [ np.nan, 'Alarm Status'],
                    'bps_breaker': [ 'ADC', 'Breaker status'],
                    'bps_i12': [ 'ADC', 'Current of 12V line (0 to 1023 ADC raw)'],
                    'bps_i375': [ 'ADC', 'Current of 375V line (0 to 1023 ADC raw)'],
                    'bps_iback': [ 'ADC', 'Backbone current (0 to 1023 ADC raw)'],
                    'bps_v375': [ 'ADC', 'Voltage of 375V line (0 to 1023 ADC raw)'],
                    'bps_v5': [ 'ADC', 'Voltage of 5V line (0 to 1023 ADC raw)'],
                    'cable_rtt': [ 's', 'Cable roundtrip time in ps'],
                    'edfa_emi': [ np.nan, 'Emission'],
                    'edfa_iop': [ 'dBm', 'Input Optical Power (in milli dBm)'],
                    'edfa_oop': [ 'dBm', 'Output Optical Power (in milli dBm)'],
                    'edfa_pumpdc': [ 'A', 'Doide Pump Current (in micro A)'],
                    'edfa_setp_rd': [ 'dBm', 'Monitored Power Setpoint for APC mode (in milli dBm)'],
                    'edfa_setp_wr': [ 'dBm', 'Settings for Power Setpoint (in dBm)'],
                    'edfa_sn': [ np.nan, 'Serial number (Ascii string)'],
                    'edfa_temp': [ 'C', 'Temperature (in milli degrees celsius)'],
                    'hr_veto_ena_ch': [ '1', 'Bitmask to enable high rate veto on PMTs'],
                    'hr_veto_thres': [ np.nan, 'Threshold in hits per timeslice'],
                    'mulhit_ena_ch': [ '1', 'Bitmask to enable multi-hits on PMTs'],
                    'pwr_meas': [ np.nan, 'power measurment'],
                    'rx_input_power': [ 'W', 'SFP receiver input power'],
                    'stmach_pktsize': [ np.nan, np.nan],
                    'sys_temp_hum': [ np.nan, 'Mixed array of temperature and humidity (0=FPGA,1=CLB,2=CLB_hum,3=POWER,4=WR,5=SFP)'],
                    'time_slice_dur': [ 's', 'Timeslice duration'],
                    'tx_output_power': [ 'W', 'SFP transmitter power ']
}

        #print( "Very Short SC parameter list: {0}".format( TEST_SC_PARAMETER_LIST) )
        #print( "Short SC parameter list: {0}".format( SHORT_SC_PARAMETER_LIST) )
        #print( "Long  SC parameter list: {0}".format( SC_PARAMETER_LIST) )



#!usr/bin/env python
# -*- coding: utf-8 -*-
# Filename: setup.py
"""
DOMAuto setup script.

"""

from setuptools import setup, find_packages

PACKAGE_NAME = 'domauto'
URL = 'https://git.km3net.de/jseneca/dom_autopsy'
DESCRIPTION = 'The domauto project. Automatic DOM status plotting and autopsy'
__author__ = 'Jordan Seneca'
__email__ = 'jseneca@km3net.de'

with open('requirements-dev.txt') as fobj:
    REQUIREMENTS = [l.strip() for l in fobj.readlines()]

setup(
    name=PACKAGE_NAME,
    url=URL,
    description= DESCRIPTION,
    author=__author__,
    author_email=__email__,
    packages=find_packages(),
    include_package_data=True,
    platforms='any',
    setup_requires=[ 'setuptools_scm', 'wheel' ],
    use_scm_version=True,
    install_requires=REQUIREMENTS,
    python_requires='>=3.0',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'Programming Language :: Python',
    ],
    entry_points = {
        'console_scripts': ['domauto=domauto.command_line:main'],
        },
    test_suite='nose.collector',
    tests_require=['nose'],
)



import os, glob

from .plotlib import *
from .iolib   import *
from .SC_PARAMETERS import *

import km3pipe


class scp_plotter:

    def __init__( self,
                  verbose = 0):

        self.verbose = verbose
        if self.verbose >= 2: print("Fetching database streams...", end='')
        self.sds = km3pipe.db.StreamDS()
        if self.verbose >= 2: print(" done.")
        
        
    def fetch( self,
               DETECTOR,
               PARAM_NAME,
               MINRUN,
               MAXRUN ):


        scp_dataframe = self.sds.get( 'datalognumbers',
                                      detid          = DETECTOR,
                                      minrun         = MINRUN,
                                      maxrun         = MAXRUN,
                                      parameter_name = PARAM_NAME )
        
        try:
            if scp_dataframe.size < 1:
                print("ERROR: No data found for detector {0}, runs {1}-{2} parameter {3}.".format( DETECTOR, MINRUN, MAXRUN, PARAM_NAME ) )
                raise ValueError
        except AttributeError:
            print("ERROR: No data found for detector {0}, runs {1}-{2} parameter {3}.".format( DETECTOR, MINRUN, MAXRUN, PARAM_NAME ) )
            raise ValueError

        return scp_dataframe


    
        
    def plot( self,
              KM3NET_DETECTOR,
              MINRUN,
              MAXRUN,
              OUTPATH = None,
              PARAM_NAME    = "",
              PARAM_UNIT    = "",
              PARAM_MEANING = "",
              ACCENT = [],
              HISTOGRAM = False,
              SCATTER   = False,
              CACHE     = False,
              REWRITE   = False,
              SAVE_FIG  = True ):
        """
        Script to create plots of a given detector for a given parameter
        
        """
                
        DETECTOR        = DETID_to_OID( KM3NET_DETECTOR, self.sds )
        DETECTOR_TYPE   = DETECTOR.split("_")[-1]

        print("Fetching CLB map...", end='')
        clbm = km3pipe.db.CLBMap( DETECTOR )
        print(" done.")

        
        if not HISTOGRAM and not SCATTER:
            print("No plotting type was chosen. Please run code again with -histogram, -scatter or both. Aborting.")
            raise ValueError

        print("Fetching data from database (this can take a while)...", end = '')        
        scp_dataframe = self.fetch( DETECTOR,
                                    PARAM_NAME,
                                    MINRUN,
                                    MAXRUN )
        print(" done.")


        
        print("Making DOM plots...", end='')

        outpath_doms = '{}{}_DOMS_SCP_{}_runs{}-{}'.format( OUTPATH,
                                                            DETECTOR,
                                                            PARAM_NAME,
                                                            MINRUN,
                                                            MAXRUN )
            
        plot_DET_DOMs( scp_dataframe,
                       clbm,
                       MINRUN,
                       MAXRUN,
                       PARAM_NAME,
                       PARAM_UNIT,
                       PARAM_MEANING,
                       OUTPATH   = outpath_doms,
                       DETECTOR = DETECTOR,
                       ACCENT = ACCENT,
                       HIST = True,
                       SCAT = True,
                       SAVE_FIG = SAVE_FIG,
                       VERBOSE = self.verbose )

        print(" done.")


        

        print( "Making DU plots..." )

        for norm, log in [(False, False), (False, True), (True, False), (True, True)]:

            print( "Plotting for " + ["no ", ""][norm] + "norm and " + ["no ", ""][log] + "log...", end = '' )

            outpath_dus = '{}{}_DUS_SCP_{}_runs{}-{}_{}{}'.format( OUTPATH,
                                                                   DETECTOR,
                                                                   PARAM_NAME,
                                                                   MINRUN,
                                                                   MAXRUN,
                                                                   ['', 'norm'][norm],
                                                                   ['', 'log'][log] )
            
            plot_DET_DUs( scp_dataframe,
                          clbm,
                          MINRUN,
                          MAXRUN,
                          PARAM_NAME,
                          PARAM_UNIT,
                          PARAM_MEANING,
                          OUTPATH = outpath_dus,
                          DETECTOR = DETECTOR,
                          ACCENT = ACCENT,
                          HIST = HISTOGRAM,
                          SCAT = SCATTER,
                          NORM = norm,
                          LOG = log,
                          SAVE_FIG = SAVE_FIG,
                          VERBOSE = self.verbose)

        print(" done.")




        
    def batch_plot( self,
                    RUN_LIST,
                    OUTPATH,
                    SCP_KEYWORD,
                    RUN_RANGE = [0, 0],
                    REWRITE = False): 

        """
        Create plots of a given detector for a list of parameters

        ENTER LIST AS -l <KM3NeT_detector_name> <run_n1> <run_n2> ...
        """

        try:
            PARAMETER_LIST = SC_PARAMETERS[ SCP_KEYWORD ]
        except KeyError:
            PARAMETER_LIST = [ param_tuple for param_tuple in SC_PARAMETERS[ 'LONG' ] if SCP_KEYWORD.upper() in param_tuple[0].upper() ]

        print( "Run list: " )
        for run in RUN_LIST: print( run )
        print( "Slow Control Parameter list: ")
        for param in PARAMETER_LIST: print( param )

        for run, omkey_list  in RUN_LIST:

            MINRUN = run.n - RUN_RANGE[0]
            MAXRUN = run.n + RUN_RANGE[1] if RUN_RANGE[1] else run.n

            for parameter in PARAMETER_LIST:

                print("\n\n- - - running plot_scp.py for {0} {1}-{2} with parameter {3} - - -\n".format( run.det_id, MINRUN, MAXRUN, parameter[0] ) )

                try:
                    self.plot( run.det_id,
                               MINRUN,
                               MAXRUN,
                               OUTPATH,
                               PARAM_NAME    = parameter[0],
                               PARAM_UNIT    = parameter[1],
                               PARAM_MEANING = parameter[2],
                               ACCENT    = omkey_list,
                               HISTOGRAM = True,
                               SCATTER   = True,
                               CACHE     = False,
                               REWRITE   = REWRITE,
                               SAVE_FIG  = True )
                except ValueError:
                    print( " ... continuing." )
                    continue
        print(" done.")


